#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from scipy.interpolate import UnivariateSpline
import collections


''' Copyright (c) 2016, Linear Technology Corp.(LTC)
All rights reserved.

Linear Technology Confidential - For Customer Use Only

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. '''


#Generated on 2016-07-27

class LTC2937(collections.Mapping):
    '''API for the LTC2937 Programmable Six Channel Sequencer and Voltage Supervisor with EEPROM.

    Each bit field is read and written as an attribute of the class instance.
    Bit fields are changed in place with a read-modify-write algorithm to avoid clearing adjacent data in shared registers.
    When multiple bit fields are stored within a single command code, an additional attribute of the class instance exists to allow reads and writes to the full register.
    Presets (enumerations) and formats (transformation functions between integers and real-word units) are applied in both directions for interactive use, but can be disabled.'''
    def __init__(self, read_function, write_function=None, verbose=False):
        '''The user must supply appropriate functions to read from and write to the I²C/SMBus hardware.
        read_function should take arguments (addr_7bit, command_code) and return contents of LTC2937 register at command_code.
        write_function should take arguments (addr_7bit, command_code, data) and write data to LTC2937 register at command_code.
        Set verbose argument to True to enable printing of intermediate results of masking, shifting and data transformation operations.
        '''
        object.__setattr__(self, 'verbose', verbose)
        object.__setattr__(self, 'read_function', read_function)
        object.__setattr__(self, 'write_function', write_function)
        object.__setattr__(self, 'addr_7bit', 0x36)
        object.__setattr__(self, 'word_size', 16)
        self._update_xml_formatters()
    def get_status(self):
        '''Returns dictionary containing current value of all readable registers.'''
        ret_dict = {}
        for k in self.keys():
            try:
                ret_dict[k] = self[k]
            except LTC2937_APIException:
                #bit field not readable
                pass 
        return ret_dict
    def print_status(self):
        '''Prints current value of all readable registers.'''
        status_dict = self.get_status()
        max_key = max([len(k) for k in status_dict.keys()])
        for k,v in sorted(status_dict.items(), key=lambda (k,v): k):
            print("{}	{}".format(k,v).expandtabs(max_key+4).replace(" ","."))
    def set_device_address(self, addr_7bit):
        '''Change I²C/SMBus device address of the LTC2937.
        addr_7bit is the right-justified 7-bit address with the R/W̅ bit omitted.'''
        assert isinstance(addr_7bit, int)
        assert addr_7bit < 0x80
        self.addr_7bit = addr_7bit
    def get_active_format(self, bf_name):
        '''Returns string name of format currently active for field bf_name.'''
        return self._register_map[bf_name]['active_format']
    def set_active_format(self, bf_name, format_name, force=False):
        '''Changes currently active format for field bf_name to format_name.
        If format_name is not in list of allowed_formats, set force argument to True to disable check.
        '''
        if format_name is None or format_name.lower() == 'none':
            self._register_map[bf_name]['active_format'] = 'None'
        elif format_name in self._register_map[bf_name]['allowed_formats'] or force:
            self._register_map[bf_name]['active_format'] = format_name
        else:
            self._error('Format {} not allowed for field {}'.format(format_name,bf_name))
    def enable_read_presets(self,bf_name,enabled=True):
        '''Enable presets (enumerations) for a single bit field bf_name.'''
        self._register_map['bf_nameread_presets_enabled'] = enabled
    def disable_read_presets(self,bf_name,enabled=False):
        '''Disable presets (enumerations) for a single bit field bf_name.'''
        self.enable_read_presets(bf_name,enabled)
    def disable_binary_read_presets(self):
        '''Disable presets (enumerations) for all single-bit bit fields.'''
        for bf_name in self._register_map.keys():
            if len(self._register_map[bf_name]['size']) == 1:
                self.disable_read_presets(bf_name)
    def get_formats(self, bf_name):
        '''Returns tuple of formats defined for bit field bf_name.'''
        return tuple(self._register_map[bf_name]['allowed_formats'])
    def create_format(self, format_name, format_function, unformat_function, signed=False, description=None):
        '''Create a new format definition or modify an existing definition.

        format_function should take a single argument of integer raw data from the register and return a version of the data scaled to appropriate units.
        unformat_function should take a single argument of data in real units and return an integer version of the data scaled to the register LSB weight.
        If the data is signed in two's-complement format, set signed=True.
        After creating format, use set_active_format method to make the new format active.
        '''
        self._formatters[format_name] = {'format': format_function, 'unformat': unformat_function, 'description': description, 'signed': signed}
    def set_constant(self, constant, value):
        '''Sets the constants found in the datasheet used by the formatters to convert from real world values to digital value and back.'''
        self._constants[constant] = value
        self._update_xml_formatters()
    def get_constant(self,constant):
        '''Gets the constants found in the datasheet used by the formatters to convert from real world values to digital value and back.'''
        return self._constants[constant]
    def list_constants(self):
        '''Returns a dictionary of constants found in the datasheet used by the formatters to convert from real world values to digital value and back.'''
        return self._constants
    def disable_presets_and_formats(self):
        '''Remove all presets and formats.

        Read and write data will be passed through unmodified, except for masking and shifting.
        The dictionary of presets and formats is shared between all instances of the class and any other instances will reflect this change.
        This is permanent for the duration of the Python session.'''
        for register in self._register_map.values():
            register['presets'] = []
            register['active_format'] = 'None'
    def _error(self, message):
        '''Raises exceptions if bad data is passed to API.
        Bus Read and Write errors must be raised by the user-supplied functions.'''
        raise LTC2937_APIException(message)
    def __str__(self):
        return 'LTC2937 API instance at address 0x36'
    def __repr__(self):
        return self.__str__()
    def __iter__(self):
        return iter(self._register_map.viewkeys())
    def __getitem__(self,key):
        return getattr(self,key)
    def __setitem__(self,key,value):
        self.__setattr__(key,value)
    def __len__(self):
        return len(self._register_map)
    def __setattr__(self, name, value):
        '''Override attribute storage mechanism to prevent writes to mis-spelled bit fields.'''
        try:
            #First try writes to bit field property decorator
            LTC2937.__dict__[name].fset(self, value)
        except KeyError as e:
            if getattr(self, name, None) is not None:
                #Then try writing to existing conventional instance attributes
                object.__setattr__(self, name, value)
            else:
                #Finally, prevent creation of new instance attributes
                self._error('Attempted write to non-existant field: {}.'.format(name))
    def _read_bf(self, bf_name):
        return self._format(bf_name,self._extract(self.read_function(self.addr_7bit, self._register_map[bf_name]['command_code']),self._register_map[bf_name]['size'],self._register_map[bf_name]['offset']))
    def _write_bf(self, bf_name, data):
        if self._register_map[bf_name]['size'] == self.word_size: #write-in-place
            data = self._unformat(bf_name,data)
            if data >= (1<<self.word_size) or data < 0:
                self._error('Data:{} does not fit in field:{} of size:{}.'.format(hex(data),bf_name,self.word_size))
            self.write_function(self.addr_7bit, self._register_map[bf_name]['command_code'],data)
        else: #read-modify-write
            self.write_function(self.addr_7bit, self._register_map[bf_name]['command_code'],self._pack(self._unformat(bf_name,data),self._register_map[bf_name]['size'],self._register_map[bf_name]['offset'],self.read_function(self.addr_7bit, self._register_map[bf_name]['command_code'])))
    def _extract(self, data, size, offset):
        result = (data >> offset) & ((1<<size)-1)
        if self.verbose:
            print('Extracting data:{}, size:{}, offset:{} from raw data:{:#018b}.'.format(bin(result),size,offset,data))
        return result
    def _pack(self, data, size, offset, old_register_contents=0):
        if self.verbose:
            print('Packing new data:{}, size:{}, offset:{} into {:#018b}'.format(bin(data),size,offset,old_register_contents))
        mask = ((1<<self.word_size)-1) ^ (((1<<size)-1)<<offset)
        masked = old_register_contents & mask
        if data >= (1<<size) or data < 0:
            self._error('Data:{} does not fit in field of size:{}.'.format(hex(data),size))
        data = (data<<offset) | masked
        if self.verbose:
            print('mask:{:#018b}, masked_old:{:#018b}, merged:{:#018b}'.format(mask,masked,data))
        return data
    def _transform_from_points(self, xlist, ylist, direction):
        '''Used internally to convert from register values to real world values and back again.'''
        x_evaled = []
        y_evaled = []
        only_constants = {}
        only_constants.update(self._constants)
        only_constants = {key:float(value) for key, value in self._constants.iteritems()}
        for xpoint in xlist:
            x_evaled.append(eval(xpoint, only_constants))
        for ypoint in ylist:
            y_evaled.append(eval(ypoint, only_constants))
        if direction == "format":
            z = sorted(zip(x_evaled, y_evaled), key = lambda x: x[0])
            return lambda x: float(UnivariateSpline(x = zip(*z)[0], y = zip(*z)[1], k=1, s = 0)(x))
        elif direction == "unformat":
            z = sorted(zip(x_evaled, y_evaled), key = lambda x: x[1])
            return lambda x: int(round(UnivariateSpline(x = zip(*z)[1], y = zip(*z)[0], k=1, s = 0)(x)))
        else:
            print("'transform_from_points()' requires one of either: 'format' or 'unformat'")
            return
    def _format(self, bf_name, data):
        if self._register_map[bf_name]['read_presets_enabled']:
            for (preset,value) in self._register_map[bf_name]['presets']:
                    if (data == value):
                        if self.verbose:
                            print('Read matched preset {} with value {}'.format(preset,value))
                        return preset
        if self.verbose:
            print('Applying format: {}'.format(self._register_map[bf_name]['active_format']))
        if self._formatters[self._register_map[bf_name]['active_format']]['signed']:
            data = self._twosComplementToSigned(data, self._register_map[bf_name]['size'])
        return self._formatters[self._register_map[bf_name]['active_format']]['format'](data)
    def _unformat(self, bf_name, data):
        for (preset,value) in self._register_map[bf_name]['presets']:
                if (data == preset):
                    if self.verbose:
                        print('Write matched preset {} with value {}'.format(preset,value))
                    return value
        if self.verbose:
            print('Un-applying format: {}'.format(self._register_map[bf_name]['active_format']))
        data = self._formatters[self._register_map[bf_name]['active_format']]['unformat'](data)
        assert isinstance(data, int)
        if self._formatters[self._register_map[bf_name]['active_format']]['signed']:
            data = self._signedToTwosComplement(data, self._register_map[bf_name]['size'])
        else:
            if data < 0:
                print("WARNING: unformatted data {} clamped to fit bitfield".format(data))
                data = 0
            elif data > 2**self._register_map[bf_name]['size'] - 1:
                print("WARNING: unformatted data {} clamped to fit bitfield".format(data))
                data = 2**self._register_map[bf_name]['size'] - 1
        return data
    def _signedToTwosComplement(self, signed, bitCount):
        '''take python int and convert to two's complement representation using specified number of bits'''
        if signed > 2**(bitCount-1) - 1:
            print("WARNING: unformatted data {} clamped to fit signed bitfield".format(signed))
            signed = 2**(bitCount-1) - 1
        elif signed < -2**(bitCount-1):
            print("WARNING: unformatted data {} clamped to fit signed bitfield".format(signed))
            signed = -2**(bitCount-1)
        if signed < 0:
            if self.verbose:
                print("Converting negative number:{} of size:{} to two's complement.".format(signed, bitCount))
            signed += 2**bitCount
            signed &= 2**bitCount-1
        return signed
    def _twosComplementToSigned(self, binary, bitCount):
        '''take two's complement number with specified number of bits and convert to python int representation'''
        assert binary <= 2**bitCount - 1
        assert binary >= 0
        if binary >= 2**(bitCount-1):
            if self.verbose:
                print('Converting binary number:{} of size:{} to negative int.'.format(binary, bitCount))
            binary -= 2**bitCount
        return binary
    def _update_xml_formatters(self):
        self.create_format( format_name = 'None',
                            format_function = lambda x:x,
                            unformat_function = lambda x:x,
                            signed = False,
                            description = '''No formatting applied to data.''')  
        for fmt_name in self._xml_formats:
            xlist, ylist = zip(*self._xml_formats[fmt_name]["points"])
            self.create_format( format_name = fmt_name,
                                format_function = self._transform_from_points(xlist, ylist, "format"),
                                unformat_function = self._transform_from_points(xlist, ylist, "unformat"),
                                signed = self._xml_formats[fmt_name]["signed"],
                                description = self._xml_formats[fmt_name]["description"])


    device_key = property(fget=lambda self: self._read_bf('device_key'),
                                         fset=lambda self,data: self._write_bf('device_key',data),
                                         doc='''Must match against programmed word to change lock bit. (14 bits, R/W)
                                                 Preset 'DEFAULT': 3754''')
    hw_lock_bit = property(fget=lambda self: self._read_bf('hw_lock_bit'),
                                         fset=lambda self,data: self._write_bf('hw_lock_bit',data),
                                         doc='''State of external WP Input. No register writes are possible when locked. (1 bit, R/W)
                                                 Preset 'UNLOCKED': 0
                                                 Preset 'LOCKED': 1''')
    sw_lock_bit = property(fget=lambda self: self._read_bf('sw_lock_bit'),
                                         fset=lambda self,data: self._write_bf('sw_lock_bit',data),
                                         doc='''Write lock bit. No register writes are possible when locked. (1 bit, R/W)
                                                 Preset 'UNLOCKED': 0
                                                 Preset 'LOCKED': 1''')
    WRITE_PROTECTION_CMD = property(fget=lambda self: self._read_bf('WRITE_PROTECTION_CMD'),
                                         fset=lambda self,data: self._write_bf('WRITE_PROTECTION_CMD',data),
                                         doc='''Contains lock key code and lock bit to prevent accidental changes.''')
    SPECIAL_LOT = property(fget=lambda self: self._read_bf('SPECIAL_LOT'),
                                         fset=lambda self,data: self._write_bf('SPECIAL_LOT',data),
                                         doc='''Returns two bytes indicating a factory special lot number. Users may over write the register with their own information. (16 bits, R/W)''')
    on_state = property(fget=lambda self: self._read_bf('on_state'),
                                         fset=lambda self,data: self._write_bf('on_state',data),
                                         doc='''Internal ON status (1 bit, R/W)
                                                 Preset 'INTERNAL_ON_LOW': 0
                                                 Preset 'INTERNAL_ON_HIGH': 1''')
    i2c_margin = property(fget=lambda self: self._read_bf('i2c_margin'),
                                         fset=lambda self,data: self._write_bf('i2c_margin',data),
                                         doc='''RSTB disable (1 bit, R/W)
                                                 Preset 'RSTB_OPERATES_NORMALLY': 0
                                                 Preset 'RSTB_PULLS_HIGH': 1''')
    discharge_start = property(fget=lambda self: self._read_bf('discharge_start'),
                                         fset=lambda self,data: self._write_bf('discharge_start',data),
                                         doc='''Require or Not Require that all sequenced Voltage Monitor Channels are discharged below their thresholds before sequencing up. (1 bit, R/W)
                                                 Preset 'DISCHARGE_NOT_REQUIRED': 0
                                                 Preset 'DISCHARGE_REQUIRED': 1''')
    i2c_on_off = property(fget=lambda self: self._read_bf('i2c_on_off'),
                                         fset=lambda self,data: self._write_bf('i2c_on_off',data),
                                         doc='''Software/I2C controlled Sequencing. (1 bit, R/W)
                                                 Preset 'SEQUENCE_DOWN': 0
                                                 Preset 'SEQUENCE_UP': 1''')
    i2c_on_off_mask = property(fget=lambda self: self._read_bf('i2c_on_off_mask'),
                                         fset=lambda self,data: self._write_bf('i2c_on_off_mask',data),
                                         doc='''serial bus on_off control mask (1 bit, R/W)
                                                 Preset 'I2C_IGNORE': 0
                                                 Preset 'I2C_LISTEN': 1''')
    on_input_mask = property(fget=lambda self: self._read_bf('on_input_mask'),
                                         fset=lambda self,data: self._write_bf('on_input_mask',data),
                                         doc='''Determines how part responds to the ON input. (1 bit, R/W)
                                                 Preset 'ON_PIN_IGNORE': 0	 part ignores the ON input
                                                 Preset 'ON_PIN_LISTEN': 1	 part requires the ON input to sequence''')
    on_polarity = property(fget=lambda self: self._read_bf('on_polarity'),
                                         fset=lambda self,data: self._write_bf('on_polarity',data),
                                         doc='''Determines polarity of the ON Input. ACTIVEHIGH = Pull ON high to sequence up. ACTIVELOW = Pull ON low to sequence up. (1 bit, R/W)
                                                 Preset 'ACTIVE_HIGH': 0
                                                 Preset 'ACTIVE_LOW': 1''')
    seq_down_qual = property(fget=lambda self: self._read_bf('seq_down_qual'),
                                         fset=lambda self,data: self._write_bf('seq_down_qual',data),
                                         doc='''Selects Time or Event based Sequence Down. (1 bit, R/W)
                                                 Preset 'DISCHARGE_BASED': 0
                                                 Preset 'TIME_BASED': 1''')
    ON_OFF_CONTROL_CMD = property(fget=lambda self: self._read_bf('ON_OFF_CONTROL_CMD'),
                                         fset=lambda self,data: self._write_bf('ON_OFF_CONTROL_CMD',data),
                                         doc='''Configures the combination of ON input and/or I2C inputs needed to control sequencing. Also, activates margin mode of operation.''')
    v6_range = property(fget=lambda self: self._read_bf('v6_range'),
                                         fset=lambda self,data: self._write_bf('v6_range',data),
                                         doc='''Monitor range setting for Voltage Monitor 6. (2 bits, R/W)
                                                 Preset 'HIGH_RANGE': 0
                                                 Preset 'LOW_RANGE': 1
                                                 Preset 'POSITIVE_ADJUSTABLE': 2
                                                 Preset 'NEGATIVE_ADJUSTABLE': 3''')
    v5_range = property(fget=lambda self: self._read_bf('v5_range'),
                                         fset=lambda self,data: self._write_bf('v5_range',data),
                                         doc='''Monitor range setting for Voltage Monitor 5. (2 bits, R/W)
                                                 Preset 'HIGH_RANGE': 0
                                                 Preset 'LOW_RANGE': 1
                                                 Preset 'POSITIVE_ADJUSTABLE': 2
                                                 Preset 'NEGATIVE_ADJUSTABLE': 3''')
    v4_range = property(fget=lambda self: self._read_bf('v4_range'),
                                         fset=lambda self,data: self._write_bf('v4_range',data),
                                         doc='''Monitor range setting for Voltage Monitor 4. (2 bits, R/W)
                                                 Preset 'HIGH_RANGE': 0
                                                 Preset 'LOW_RANGE': 1
                                                 Preset 'POSITIVE_ADJUSTABLE': 2
                                                 Preset 'NEGATIVE_ADJUSTABLE': 3''')
    v3_range = property(fget=lambda self: self._read_bf('v3_range'),
                                         fset=lambda self,data: self._write_bf('v3_range',data),
                                         doc='''Monitor range setting for Voltage Monitor 3. (2 bits, R/W)
                                                 Preset 'HIGH_RANGE': 0
                                                 Preset 'LOW_RANGE': 1
                                                 Preset 'POSITIVE_ADJUSTABLE': 2
                                                 Preset 'NEGATIVE_ADJUSTABLE': 3''')
    v2_range = property(fget=lambda self: self._read_bf('v2_range'),
                                         fset=lambda self,data: self._write_bf('v2_range',data),
                                         doc='''Monitor range setting for Voltage Monitor 2. (2 bits, R/W)
                                                 Preset 'HIGH_RANGE': 0
                                                 Preset 'LOW_RANGE': 1
                                                 Preset 'POSITIVE_ADJUSTABLE': 2
                                                 Preset 'NEGATIVE_ADJUSTABLE': 3''')
    v1_range = property(fget=lambda self: self._read_bf('v1_range'),
                                         fset=lambda self,data: self._write_bf('v1_range',data),
                                         doc='''Monitor range setting for Voltage Monitor 1. (2 bits, R/W)
                                                 Preset 'HIGH_RANGE': 0
                                                 Preset 'LOW_RANGE': 1
                                                 Preset 'POSITIVE_ADJUSTABLE': 2
                                                 Preset 'NEGATIVE_ADJUSTABLE': 3''')
    V_RANGE_CMD = property(fget=lambda self: self._read_bf('V_RANGE_CMD'),
                                         fset=lambda self,data: self._write_bf('V_RANGE_CMD',data),
                                         doc='''''')
    ov_threshold_1 = property(fget=lambda self: self._read_bf('ov_threshold_1'),
                                         fset=lambda self,data: self._write_bf('ov_threshold_1',data),
                                         doc='''Specify V1 OV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    uv_threshold_1 = property(fget=lambda self: self._read_bf('uv_threshold_1'),
                                         fset=lambda self,data: self._write_bf('uv_threshold_1',data),
                                         doc='''Specify V1 UV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    V_THRESHOLD_1_CMD = property(fget=lambda self: self._read_bf('V_THRESHOLD_1_CMD'),
                                         fset=lambda self,data: self._write_bf('V_THRESHOLD_1_CMD',data),
                                         doc='''''')
    ov_threshold_2 = property(fget=lambda self: self._read_bf('ov_threshold_2'),
                                         fset=lambda self,data: self._write_bf('ov_threshold_2',data),
                                         doc='''Specify V2 OV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    uv_threshold_2 = property(fget=lambda self: self._read_bf('uv_threshold_2'),
                                         fset=lambda self,data: self._write_bf('uv_threshold_2',data),
                                         doc='''Specify V2 UV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    V_THRESHOLD_2_CMD = property(fget=lambda self: self._read_bf('V_THRESHOLD_2_CMD'),
                                         fset=lambda self,data: self._write_bf('V_THRESHOLD_2_CMD',data),
                                         doc='''''')
    ov_threshold_3 = property(fget=lambda self: self._read_bf('ov_threshold_3'),
                                         fset=lambda self,data: self._write_bf('ov_threshold_3',data),
                                         doc='''Specify V3 OV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    uv_threshold_3 = property(fget=lambda self: self._read_bf('uv_threshold_3'),
                                         fset=lambda self,data: self._write_bf('uv_threshold_3',data),
                                         doc='''Specify V3 UV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    V_THRESHOLD_3_CMD = property(fget=lambda self: self._read_bf('V_THRESHOLD_3_CMD'),
                                         fset=lambda self,data: self._write_bf('V_THRESHOLD_3_CMD',data),
                                         doc='''''')
    ov_threshold_4 = property(fget=lambda self: self._read_bf('ov_threshold_4'),
                                         fset=lambda self,data: self._write_bf('ov_threshold_4',data),
                                         doc='''Specify V4 OV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    uv_threshold_4 = property(fget=lambda self: self._read_bf('uv_threshold_4'),
                                         fset=lambda self,data: self._write_bf('uv_threshold_4',data),
                                         doc='''Specify V4 UV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    V_THRESHOLD_4_CMD = property(fget=lambda self: self._read_bf('V_THRESHOLD_4_CMD'),
                                         fset=lambda self,data: self._write_bf('V_THRESHOLD_4_CMD',data),
                                         doc='''''')
    ov_threshold_5 = property(fget=lambda self: self._read_bf('ov_threshold_5'),
                                         fset=lambda self,data: self._write_bf('ov_threshold_5',data),
                                         doc='''Specify V5 OV Threshold.. (8 bits, R/W)
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    uv_threshold_5 = property(fget=lambda self: self._read_bf('uv_threshold_5'),
                                         fset=lambda self,data: self._write_bf('uv_threshold_5',data),
                                         doc='''Specify V5 UV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    V_THRESHOLD_5_CMD = property(fget=lambda self: self._read_bf('V_THRESHOLD_5_CMD'),
                                         fset=lambda self,data: self._write_bf('V_THRESHOLD_5_CMD',data),
                                         doc='''''')
    ov_threshold_6 = property(fget=lambda self: self._read_bf('ov_threshold_6'),
                                         fset=lambda self,data: self._write_bf('ov_threshold_6',data),
                                         doc='''Specify V6 OV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    uv_threshold_6 = property(fget=lambda self: self._read_bf('uv_threshold_6'),
                                         fset=lambda self,data: self._write_bf('uv_threshold_6',data),
                                         doc='''Specify V6 UV Threshold. (8 bits, R/W)
                                                 Format: Volts_to_TE_HR	convert a voltage value to Threshold Encoding for High Range
                                                 Format: Volts_to_TE_LR	convert a voltage value to Threshold Encoding for Low Range
                                                 Format: Volts_to_TE_AR	convert a voltage value to Threshold Encoding for Adjustable Range''')
    V_THRESHOLD_6_CMD = property(fget=lambda self: self._read_bf('V_THRESHOLD_6_CMD'),
                                         fset=lambda self,data: self._write_bf('V_THRESHOLD_6_CMD',data),
                                         doc='''''')
    ton_max_1 = property(fget=lambda self: self._read_bf('ton_max_1'),
                                         fset=lambda self,data: self._write_bf('ton_max_1',data),
                                         doc='''Sets the maximum voltage monitor rise time for Channel 1. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '160us': 1
                                                 Preset '640us': 2
                                                 Preset '2_6ms': 3
                                                 Preset '10_2ms': 4
                                                 Preset '41ms': 5
                                                 Preset '164ms': 6
                                                 Preset '655ms': 7''')
    ton_delay_1 = property(fget=lambda self: self._read_bf('ton_delay_1'),
                                         fset=lambda self,data: self._write_bf('ton_delay_1',data),
                                         doc='''Sets time delay between the start of V1's selected sequence position and the release of EN1. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TON_TIMERS_1_CMD = property(fget=lambda self: self._read_bf('TON_TIMERS_1_CMD'),
                                         fset=lambda self,data: self._write_bf('TON_TIMERS_1_CMD',data),
                                         doc='''''')
    ton_max_2 = property(fget=lambda self: self._read_bf('ton_max_2'),
                                         fset=lambda self,data: self._write_bf('ton_max_2',data),
                                         doc='''Sets the maximum voltage monitor rise time for Channel 2. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '160us': 1
                                                 Preset '640us': 2
                                                 Preset '2_6ms': 3
                                                 Preset '10_2ms': 4
                                                 Preset '41ms': 5
                                                 Preset '164ms': 6
                                                 Preset '655ms': 7''')
    ton_delay_2 = property(fget=lambda self: self._read_bf('ton_delay_2'),
                                         fset=lambda self,data: self._write_bf('ton_delay_2',data),
                                         doc='''Sets time delay between the start of V2's selected sequence position and the release of EN2. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TON_TIMERS_2_CMD = property(fget=lambda self: self._read_bf('TON_TIMERS_2_CMD'),
                                         fset=lambda self,data: self._write_bf('TON_TIMERS_2_CMD',data),
                                         doc='''''')
    ton_max_3 = property(fget=lambda self: self._read_bf('ton_max_3'),
                                         fset=lambda self,data: self._write_bf('ton_max_3',data),
                                         doc='''Sets the maximum voltage monitor rise time for Channel 3. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '160us': 1
                                                 Preset '640us': 2
                                                 Preset '2_6ms': 3
                                                 Preset '10_2ms': 4
                                                 Preset '41ms': 5
                                                 Preset '164ms': 6
                                                 Preset '655ms': 7''')
    ton_delay_3 = property(fget=lambda self: self._read_bf('ton_delay_3'),
                                         fset=lambda self,data: self._write_bf('ton_delay_3',data),
                                         doc='''Sets time delay between the start of V3's selected sequence position and the release of EN3. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TON_TIMERS_3_CMD = property(fget=lambda self: self._read_bf('TON_TIMERS_3_CMD'),
                                         fset=lambda self,data: self._write_bf('TON_TIMERS_3_CMD',data),
                                         doc='''''')
    ton_max_4 = property(fget=lambda self: self._read_bf('ton_max_4'),
                                         fset=lambda self,data: self._write_bf('ton_max_4',data),
                                         doc='''Sets the maximum voltage monitor rise time for Channel 4. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '160us': 1
                                                 Preset '640us': 2
                                                 Preset '2_6ms': 3
                                                 Preset '10_2ms': 4
                                                 Preset '41ms': 5
                                                 Preset '164ms': 6
                                                 Preset '655ms': 7''')
    ton_delay_4 = property(fget=lambda self: self._read_bf('ton_delay_4'),
                                         fset=lambda self,data: self._write_bf('ton_delay_4',data),
                                         doc='''Sets time delay between the start of V4's selected sequence position and the release of EN4. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TON_TIMERS_4_CMD = property(fget=lambda self: self._read_bf('TON_TIMERS_4_CMD'),
                                         fset=lambda self,data: self._write_bf('TON_TIMERS_4_CMD',data),
                                         doc='''''')
    ton_max_5 = property(fget=lambda self: self._read_bf('ton_max_5'),
                                         fset=lambda self,data: self._write_bf('ton_max_5',data),
                                         doc='''Sets the maximum voltage monitor rise time for Channel 5. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '160us': 1
                                                 Preset '640us': 2
                                                 Preset '2_6ms': 3
                                                 Preset '10_2ms': 4
                                                 Preset '41ms': 5
                                                 Preset '164ms': 6
                                                 Preset '655ms': 7''')
    ton_delay_5 = property(fget=lambda self: self._read_bf('ton_delay_5'),
                                         fset=lambda self,data: self._write_bf('ton_delay_5',data),
                                         doc='''Sets time delay between the start of V5's selected sequence position and the release of EN5. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TON_TIMERS_5_CMD = property(fget=lambda self: self._read_bf('TON_TIMERS_5_CMD'),
                                         fset=lambda self,data: self._write_bf('TON_TIMERS_5_CMD',data),
                                         doc='''''')
    ton_max_6 = property(fget=lambda self: self._read_bf('ton_max_6'),
                                         fset=lambda self,data: self._write_bf('ton_max_6',data),
                                         doc='''Sets the maximum voltage monitor rise time for Channel 6. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '160us': 1
                                                 Preset '640us': 2
                                                 Preset '2_6ms': 3
                                                 Preset '10_2ms': 4
                                                 Preset '41ms': 5
                                                 Preset '164ms': 6
                                                 Preset '655ms': 7''')
    ton_delay_6 = property(fget=lambda self: self._read_bf('ton_delay_6'),
                                         fset=lambda self,data: self._write_bf('ton_delay_6',data),
                                         doc='''Sets time delay between the start of V6's selected sequence position and the release of EN6. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TON_TIMERS_6_CMD = property(fget=lambda self: self._read_bf('TON_TIMERS_6_CMD'),
                                         fset=lambda self,data: self._write_bf('TON_TIMERS_6_CMD',data),
                                         doc='''''')
    toff_max_1 = property(fget=lambda self: self._read_bf('toff_max_1'),
                                         fset=lambda self,data: self._write_bf('toff_max_1',data),
                                         doc='''Sets the maximum voltage monitor fall time for Channel 1. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '2_6ms': 1
                                                 Preset '10_2ms': 2
                                                 Preset '41ms': 3
                                                 Preset '164ms': 4
                                                 Preset '655ms': 5
                                                 Preset '2_6s': 6
                                                 Preset '10_5s': 7''')
    toff_delay_1 = property(fget=lambda self: self._read_bf('toff_delay_1'),
                                         fset=lambda self,data: self._write_bf('toff_delay_1',data),
                                         doc='''Sets time delay between the start of V1's selected sequence position and pulling EN1 low. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TOFF_TIMERS_1_CMD = property(fget=lambda self: self._read_bf('TOFF_TIMERS_1_CMD'),
                                         fset=lambda self,data: self._write_bf('TOFF_TIMERS_1_CMD',data),
                                         doc='''''')
    toff_max_2 = property(fget=lambda self: self._read_bf('toff_max_2'),
                                         fset=lambda self,data: self._write_bf('toff_max_2',data),
                                         doc='''Sets the maximum voltage monitor fall time for Channel 2. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '2_6ms': 1
                                                 Preset '10_2ms': 2
                                                 Preset '41ms': 3
                                                 Preset '164ms': 4
                                                 Preset '655ms': 5
                                                 Preset '2_6s': 6
                                                 Preset '10_5s': 7''')
    toff_delay_2 = property(fget=lambda self: self._read_bf('toff_delay_2'),
                                         fset=lambda self,data: self._write_bf('toff_delay_2',data),
                                         doc='''Sets time delay between the start of V2's selected sequence position and pulling EN2 low. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TOFF_TIMERS_2_CMD = property(fget=lambda self: self._read_bf('TOFF_TIMERS_2_CMD'),
                                         fset=lambda self,data: self._write_bf('TOFF_TIMERS_2_CMD',data),
                                         doc='''''')
    toff_max_3 = property(fget=lambda self: self._read_bf('toff_max_3'),
                                         fset=lambda self,data: self._write_bf('toff_max_3',data),
                                         doc='''Sets the maximum voltage monitor fall time for Channel 3. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '2_6ms': 1
                                                 Preset '10_2ms': 2
                                                 Preset '41ms': 3
                                                 Preset '164ms': 4
                                                 Preset '655ms': 5
                                                 Preset '2_6s': 6
                                                 Preset '10_5s': 7''')
    toff_delay_3 = property(fget=lambda self: self._read_bf('toff_delay_3'),
                                         fset=lambda self,data: self._write_bf('toff_delay_3',data),
                                         doc='''Sets time delay between the start of V3's selected sequence position and pulling EN3 low. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TOFF_TIMERS_3_CMD = property(fget=lambda self: self._read_bf('TOFF_TIMERS_3_CMD'),
                                         fset=lambda self,data: self._write_bf('TOFF_TIMERS_3_CMD',data),
                                         doc='''''')
    toff_max_4 = property(fget=lambda self: self._read_bf('toff_max_4'),
                                         fset=lambda self,data: self._write_bf('toff_max_4',data),
                                         doc='''Sets the maximum voltage monitor fall time for Channel 4. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '2_6ms': 1
                                                 Preset '10_2ms': 2
                                                 Preset '41ms': 3
                                                 Preset '164ms': 4
                                                 Preset '655ms': 5
                                                 Preset '2_6s': 6
                                                 Preset '10_5s': 7''')
    toff_delay_4 = property(fget=lambda self: self._read_bf('toff_delay_4'),
                                         fset=lambda self,data: self._write_bf('toff_delay_4',data),
                                         doc='''Sets time delay between the start of V4's selected sequence position and pulling EN4 low. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TOFF_TIMERS_4_CMD = property(fget=lambda self: self._read_bf('TOFF_TIMERS_4_CMD'),
                                         fset=lambda self,data: self._write_bf('TOFF_TIMERS_4_CMD',data),
                                         doc='''''')
    toff_max_5 = property(fget=lambda self: self._read_bf('toff_max_5'),
                                         fset=lambda self,data: self._write_bf('toff_max_5',data),
                                         doc='''Sets the maximum voltage monitor fall time for Channel 5. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '2_6ms': 1
                                                 Preset '10_2ms': 2
                                                 Preset '41ms': 3
                                                 Preset '164ms': 4
                                                 Preset '655ms': 5
                                                 Preset '2_6s': 6
                                                 Preset '10_5s': 7''')
    toff_delay_5 = property(fget=lambda self: self._read_bf('toff_delay_5'),
                                         fset=lambda self,data: self._write_bf('toff_delay_5',data),
                                         doc='''Sets time delay between the start of V5's selected sequence position and pulling EN5 low. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TOFF_TIMERS_5_CMD = property(fget=lambda self: self._read_bf('TOFF_TIMERS_5_CMD'),
                                         fset=lambda self,data: self._write_bf('TOFF_TIMERS_5_CMD',data),
                                         doc='''''')
    toff_max_6 = property(fget=lambda self: self._read_bf('toff_max_6'),
                                         fset=lambda self,data: self._write_bf('toff_max_6',data),
                                         doc='''Sets the maximum voltage monitor fall time for Channel 6. (3 bits, R/W)
                                                 Preset 'INFINITY': 0
                                                 Preset '2_6ms': 1
                                                 Preset '10_2ms': 2
                                                 Preset '41ms': 3
                                                 Preset '164ms': 4
                                                 Preset '655ms': 5
                                                 Preset '2_6s': 6
                                                 Preset '10_5s': 7''')
    toff_delay_6 = property(fget=lambda self: self._read_bf('toff_delay_6'),
                                         fset=lambda self,data: self._write_bf('toff_delay_6',data),
                                         doc='''Sets time delay between the start of V6's selected sequence position and pulling EN6 low. (13 bits, R/W)
                                                 Format: Delay_time_to_Integer	convert an Enable Delay to 13-bit integer''')
    TOFF_TIMERS_6_CMD = property(fget=lambda self: self._read_bf('TOFF_TIMERS_6_CMD'),
                                         fset=lambda self,data: self._write_bf('TOFF_TIMERS_6_CMD',data),
                                         doc='''''')
    async_on_off_1 = property(fget=lambda self: self._read_bf('async_on_off_1'),
                                         fset=lambda self,data: self._write_bf('async_on_off_1',data),
                                         doc='''Ch.1 Asynchronous enable control. Bits 9:0 have to be 0 to use. (1 bit, R/W)
                                                 Preset 'HOLD_EN1_LOW': 0
                                                 Preset 'RELEASE_EN1': 1''')
    seq_up_position_1 = property(fget=lambda self: self._read_bf('seq_up_position_1'),
                                         fset=lambda self,data: self._write_bf('seq_up_position_1',data),
                                         doc='''Specifies sequence position for channel 1. Set to 0 to remove channel from sequencing. (10 bits, R/W)''')
    SEQ_UP_POSITION_1_CMD = property(fget=lambda self: self._read_bf('SEQ_UP_POSITION_1_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_UP_POSITION_1_CMD',data),
                                         doc='''''')
    async_on_off_2 = property(fget=lambda self: self._read_bf('async_on_off_2'),
                                         fset=lambda self,data: self._write_bf('async_on_off_2',data),
                                         doc='''Ch.2 Asynchronous enable control. Bits 9:0 have to be 0 to use. (1 bit, R/W)
                                                 Preset 'HOLD_EN2_LOW': 0
                                                 Preset 'RELEASE_EN2': 1''')
    seq_up_position_2 = property(fget=lambda self: self._read_bf('seq_up_position_2'),
                                         fset=lambda self,data: self._write_bf('seq_up_position_2',data),
                                         doc='''Specifies sequence position for channel 2. Set to 0 to remove channel from sequencing. (10 bits, R/W)''')
    SEQ_UP_POSITION_2_CMD = property(fget=lambda self: self._read_bf('SEQ_UP_POSITION_2_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_UP_POSITION_2_CMD',data),
                                         doc='''''')
    async_on_off_3 = property(fget=lambda self: self._read_bf('async_on_off_3'),
                                         fset=lambda self,data: self._write_bf('async_on_off_3',data),
                                         doc='''Ch.3 Asynchronous enable control. Bits 9:0 have to be 0 to use. (1 bit, R/W)
                                                 Preset 'HOLD_EN3_LOW': 0
                                                 Preset 'RELEASE_EN3': 1''')
    seq_up_position_3 = property(fget=lambda self: self._read_bf('seq_up_position_3'),
                                         fset=lambda self,data: self._write_bf('seq_up_position_3',data),
                                         doc='''Specifies sequence position for channel 3. Set to 0 to remove channel from sequencing. (10 bits, R/W)''')
    SEQ_UP_POSITION_3_CMD = property(fget=lambda self: self._read_bf('SEQ_UP_POSITION_3_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_UP_POSITION_3_CMD',data),
                                         doc='''''')
    async_on_off_4 = property(fget=lambda self: self._read_bf('async_on_off_4'),
                                         fset=lambda self,data: self._write_bf('async_on_off_4',data),
                                         doc='''Ch.4 Asynchronous enable control. Bits 9:0 have to be 0 to use. (1 bit, R/W)
                                                 Preset 'HOLD_EN4_LOW': 0
                                                 Preset 'RELEASE_EN4': 1''')
    seq_up_position_4 = property(fget=lambda self: self._read_bf('seq_up_position_4'),
                                         fset=lambda self,data: self._write_bf('seq_up_position_4',data),
                                         doc='''Specifies sequence position for channel 4. Set to 0 to remove channel from sequencing. (10 bits, R/W)''')
    SEQ_UP_POSITION_4_CMD = property(fget=lambda self: self._read_bf('SEQ_UP_POSITION_4_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_UP_POSITION_4_CMD',data),
                                         doc='''''')
    async_on_off_5 = property(fget=lambda self: self._read_bf('async_on_off_5'),
                                         fset=lambda self,data: self._write_bf('async_on_off_5',data),
                                         doc='''Ch.5 Asynchronous enable control. Bits 9:0 have to be 0 to use. (1 bit, R/W)
                                                 Preset 'HOLD_EN5_LOW': 0
                                                 Preset 'RELEASE_EN5': 1''')
    seq_up_position_5 = property(fget=lambda self: self._read_bf('seq_up_position_5'),
                                         fset=lambda self,data: self._write_bf('seq_up_position_5',data),
                                         doc='''Specifies sequence position for channel 5. Set to 0 to remove channel from sequencing. (10 bits, R/W)''')
    SEQ_UP_POSITION_5_CMD = property(fget=lambda self: self._read_bf('SEQ_UP_POSITION_5_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_UP_POSITION_5_CMD',data),
                                         doc='''''')
    async_on_off_6 = property(fget=lambda self: self._read_bf('async_on_off_6'),
                                         fset=lambda self,data: self._write_bf('async_on_off_6',data),
                                         doc='''Ch.6 Asynchronous enable control. Bits 9:0 have to be 0 to use. (1 bit, R/W)
                                                 Preset 'HOLD_EN6_LOW': 0
                                                 Preset 'RELEASE_EN6': 1''')
    seq_up_position_6 = property(fget=lambda self: self._read_bf('seq_up_position_6'),
                                         fset=lambda self,data: self._write_bf('seq_up_position_6',data),
                                         doc='''Specifies sequence position for channel 6. Set to 0 to remove channel from sequencing. (10 bits, R/W)''')
    SEQ_UP_POSITION_6_CMD = property(fget=lambda self: self._read_bf('SEQ_UP_POSITION_6_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_UP_POSITION_6_CMD',data),
                                         doc='''''')
    active_pull_down_1 = property(fget=lambda self: self._read_bf('active_pull_down_1'),
                                         fset=lambda self,data: self._write_bf('active_pull_down_1',data),
                                         doc='''Voltage Monitor 1 active pull_down ON/OFF control. (1 bit, R/W)
                                                 Preset 'PULL_DOWN_DISABLED': 0
                                                 Preset 'PULL_DOWN_ENABLED': 1''')
    seq_down_position_1 = property(fget=lambda self: self._read_bf('seq_down_position_1'),
                                         fset=lambda self,data: self._write_bf('seq_down_position_1',data),
                                         doc='''Specifies sequence down position for channel 1. (10 bits, R/W)''')
    SEQ_DOWN_POSITION_1_CMD = property(fget=lambda self: self._read_bf('SEQ_DOWN_POSITION_1_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_DOWN_POSITION_1_CMD',data),
                                         doc='''''')
    active_pull_down_2 = property(fget=lambda self: self._read_bf('active_pull_down_2'),
                                         fset=lambda self,data: self._write_bf('active_pull_down_2',data),
                                         doc='''Voltage Monitor 2 active pull_down ON/OFF control. (1 bit, R/W)
                                                 Preset 'PULL_DOWN_DISABLED': 0
                                                 Preset 'PULL_DOWN_ENABLED': 1''')
    seq_down_position_2 = property(fget=lambda self: self._read_bf('seq_down_position_2'),
                                         fset=lambda self,data: self._write_bf('seq_down_position_2',data),
                                         doc='''Specifies sequence down position for channel 2. (10 bits, R/W)''')
    SEQ_DOWN_POSITION_2_CMD = property(fget=lambda self: self._read_bf('SEQ_DOWN_POSITION_2_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_DOWN_POSITION_2_CMD',data),
                                         doc='''''')
    active_pull_down_3 = property(fget=lambda self: self._read_bf('active_pull_down_3'),
                                         fset=lambda self,data: self._write_bf('active_pull_down_3',data),
                                         doc='''Voltage Monitor 3 active pull_down ON/OFF control. (1 bit, R/W)
                                                 Preset 'PULL_DOWN_DISABLED': 0
                                                 Preset 'PULL_DOWN_ENABLED': 1''')
    seq_down_position_3 = property(fget=lambda self: self._read_bf('seq_down_position_3'),
                                         fset=lambda self,data: self._write_bf('seq_down_position_3',data),
                                         doc='''Specifies sequence down position for channel 3. (10 bits, R/W)''')
    SEQ_DOWN_POSITION_3_CMD = property(fget=lambda self: self._read_bf('SEQ_DOWN_POSITION_3_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_DOWN_POSITION_3_CMD',data),
                                         doc='''''')
    active_pull_down_4 = property(fget=lambda self: self._read_bf('active_pull_down_4'),
                                         fset=lambda self,data: self._write_bf('active_pull_down_4',data),
                                         doc='''Voltage Monitor 4 active pull_down ON/OFF control. (1 bit, R/W)
                                                 Preset 'PULL_DOWN_DISABLED': 0
                                                 Preset 'PULL_DOWN_ENABLED': 1''')
    seq_down_position_4 = property(fget=lambda self: self._read_bf('seq_down_position_4'),
                                         fset=lambda self,data: self._write_bf('seq_down_position_4',data),
                                         doc='''Specifies sequence down position for channel 4. (10 bits, R/W)''')
    SEQ_DOWN_POSITION_4_CMD = property(fget=lambda self: self._read_bf('SEQ_DOWN_POSITION_4_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_DOWN_POSITION_4_CMD',data),
                                         doc='''''')
    active_pull_down_5 = property(fget=lambda self: self._read_bf('active_pull_down_5'),
                                         fset=lambda self,data: self._write_bf('active_pull_down_5',data),
                                         doc='''Voltage Monitor 5 active pull_down ON/OFF control. (1 bit, R/W)
                                                 Preset 'PULL_DOWN_DISABLED': 0
                                                 Preset 'PULL_DOWN_ENABLED': 1''')
    seq_down_position_5 = property(fget=lambda self: self._read_bf('seq_down_position_5'),
                                         fset=lambda self,data: self._write_bf('seq_down_position_5',data),
                                         doc='''Specifies sequence down position for channel 5. (10 bits, R/W)''')
    SEQ_DOWN_POSITION_5_CMD = property(fget=lambda self: self._read_bf('SEQ_DOWN_POSITION_5_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_DOWN_POSITION_5_CMD',data),
                                         doc='''''')
    active_pull_down_6 = property(fget=lambda self: self._read_bf('active_pull_down_6'),
                                         fset=lambda self,data: self._write_bf('active_pull_down_6',data),
                                         doc='''Voltage Monitor 6 active pull_down ON/OFF control. (1 bit, R/W)
                                                 Preset 'PULL_DOWN_DISABLED': 0
                                                 Preset 'PULL_DOWN_ENABLED': 1''')
    seq_down_position_6 = property(fget=lambda self: self._read_bf('seq_down_position_6'),
                                         fset=lambda self,data: self._write_bf('seq_down_position_6',data),
                                         doc='''Specifies sequence down position for channel 6. (10 bits, R/W)''')
    SEQ_DOWN_POSITION_6_CMD = property(fget=lambda self: self._read_bf('SEQ_DOWN_POSITION_6_CMD'),
                                         fset=lambda self,data: self._write_bf('SEQ_DOWN_POSITION_6_CMD',data),
                                         doc='''''')
    rstb_delay = property(fget=lambda self: self._read_bf('rstb_delay'),
                                         fset=lambda self,data: self._write_bf('rstb_delay',data),
                                         doc='''Sets the RSTB assertion delay. (3 bits, R/W)
                                                 Preset '0_05ms': 0
                                                 Preset '1_6ms': 1
                                                 Preset '6_4ms': 2
                                                 Preset '26ms': 3
                                                 Preset '51ms': 4
                                                 Preset '200ms': 5
                                                 Preset '410ms': 6
                                                 Preset '1640ms': 7''')
    sv_fault_enable = property(fget=lambda self: self._read_bf('sv_fault_enable'),
                                         fset=lambda self,data: self._write_bf('sv_fault_enable',data),
                                         doc='''Allows SUPERVISOR fault to pull FAULTB. (1 bit, R/W)
                                                 Preset 'SUPERVISOR_FAULT_DOES_NOT_PULL_FAULTB': 0
                                                 Preset 'SUPERVISOR_FAULT_PULLS_FAULTB': 1''')
    v6_ov_enable = property(fget=lambda self: self._read_bf('v6_ov_enable'),
                                         fset=lambda self,data: self._write_bf('v6_ov_enable',data),
                                         doc='''Maps Channel 6 OV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v5_ov_enable = property(fget=lambda self: self._read_bf('v5_ov_enable'),
                                         fset=lambda self,data: self._write_bf('v5_ov_enable',data),
                                         doc='''Maps Channel 5 OV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v4_ov_enable = property(fget=lambda self: self._read_bf('v4_ov_enable'),
                                         fset=lambda self,data: self._write_bf('v4_ov_enable',data),
                                         doc='''Maps Channel 4 OV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v3_ov_enable = property(fget=lambda self: self._read_bf('v3_ov_enable'),
                                         fset=lambda self,data: self._write_bf('v3_ov_enable',data),
                                         doc='''Maps Channel 3 OV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v2_ov_enable = property(fget=lambda self: self._read_bf('v2_ov_enable'),
                                         fset=lambda self,data: self._write_bf('v2_ov_enable',data),
                                         doc='''Maps Channel 2 OV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v1_ov_enable = property(fget=lambda self: self._read_bf('v1_ov_enable'),
                                         fset=lambda self,data: self._write_bf('v1_ov_enable',data),
                                         doc='''Maps Channel 1 OV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v6_uv_enable = property(fget=lambda self: self._read_bf('v6_uv_enable'),
                                         fset=lambda self,data: self._write_bf('v6_uv_enable',data),
                                         doc='''Maps Channel 6 UV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v5_uv_enable = property(fget=lambda self: self._read_bf('v5_uv_enable'),
                                         fset=lambda self,data: self._write_bf('v5_uv_enable',data),
                                         doc='''Maps Channel 5 UV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v4_uv_enable = property(fget=lambda self: self._read_bf('v4_uv_enable'),
                                         fset=lambda self,data: self._write_bf('v4_uv_enable',data),
                                         doc='''Maps Channel 4 UV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v3_uv_enable = property(fget=lambda self: self._read_bf('v3_uv_enable'),
                                         fset=lambda self,data: self._write_bf('v3_uv_enable',data),
                                         doc='''Maps Channel 3 UV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v2_uv_enable = property(fget=lambda self: self._read_bf('v2_uv_enable'),
                                         fset=lambda self,data: self._write_bf('v2_uv_enable',data),
                                         doc='''Maps Channel 2 UV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    v1_uv_enable = property(fget=lambda self: self._read_bf('v1_uv_enable'),
                                         fset=lambda self,data: self._write_bf('v1_uv_enable',data),
                                         doc='''Maps Channel 1 UV into RSTB output. (1 bit, R/W)
                                                 Preset 'NOT_ENABLED': 0
                                                 Preset 'ENABLED': 1''')
    RSTB_CONFIG_CMD = property(fget=lambda self: self._read_bf('RSTB_CONFIG_CMD'),
                                         fset=lambda self,data: self._write_bf('RSTB_CONFIG_CMD',data),
                                         doc='''''')
    retry_count = property(fget=lambda self: self._read_bf('retry_count'),
                                         fset=lambda self,data: self._write_bf('retry_count',data),
                                         doc='''Read the number of retries attempted. (3 bits, R/W)
                                                 Preset 'UNDEFINED': 7''')
    faultb_state = property(fget=lambda self: self._read_bf('faultb_state'),
                                         fset=lambda self,data: self._write_bf('faultb_state',data),
                                         doc='''Reports the Fault Pin state. (1 bit, R/W)
                                                 Preset 'HIGH': 1
                                                 Preset 'LOW': 0''')
    faultb_mask = property(fget=lambda self: self._read_bf('faultb_mask'),
                                         fset=lambda self,data: self._write_bf('faultb_mask',data),
                                         doc='''Determines how device responds to external faults. (1 bit, R/W)
                                                 Preset 'IGNORE_EXTERNAL_FAULTB_PULLDOWN': 0
                                                 Preset 'RESPOND_TO_EXTERNAL_FAULTB_PULLDOWN': 1''')
    freeze = property(fget=lambda self: self._read_bf('freeze'),
                                         fset=lambda self,data: self._write_bf('freeze',data),
                                         doc='''System Freeze Control. Refer to Presets for Values (1 bit, R/W)
                                                 Preset 'DO_NOT_FREEZE_ON_FAULT': 0
                                                 Preset 'FREEZE_DEVICE_STATE_ON_FAULT': 1''')
    retry_delay = property(fget=lambda self: self._read_bf('retry_delay'),
                                         fset=lambda self,data: self._write_bf('retry_delay',data),
                                         doc='''Fault Retry Delay Settings. (3 bits, R/W)
                                                 Preset '0_05ms': 0
                                                 Preset '200ms': 1
                                                 Preset '410ms': 2
                                                 Preset '820ms': 3
                                                 Preset '1_64s': 4
                                                 Preset '3_28s': 5
                                                 Preset '6_55s': 6
                                                 Preset '13_1s': 7''')
    fault_response_action = property(fget=lambda self: self._read_bf('fault_response_action'),
                                         fset=lambda self,data: self._write_bf('fault_response_action',data),
                                         doc='''Fault response action. (2 bits, R/W)
                                                 Preset 'CONTINUE_OPERATION': 0
                                                 Preset 'DISCHARGED_RETRY': 1
                                                 Preset 'DELAYED_RETRY': 2
                                                 Preset 'DISCHARGE_AND_DELAY_RETRY': 3''')
    retry_number = property(fget=lambda self: self._read_bf('retry_number'),
                                         fset=lambda self,data: self._write_bf('retry_number',data),
                                         doc='''Automatic Retry count. (3 bits, R/W)
                                                 Preset 'DO_NOT_ATTEMPT_TO_RETRY': 0
                                                 Preset 'RETRY_1_TIMES': 1
                                                 Preset 'RETRY_2_TIMES': 2
                                                 Preset 'RETRY_3_TIMES': 3
                                                 Preset 'RETRY_4_TIMES': 4
                                                 Preset 'RETRY_5_TIMES': 5
                                                 Preset 'RETRY_6_TIMES': 6
                                                 Preset 'UNLIMITED_RETRIES': 7''')
    FAULT_RESPONSE_CMD = property(fget=lambda self: self._read_bf('FAULT_RESPONSE_CMD'),
                                         fset=lambda self,data: self._write_bf('FAULT_RESPONSE_CMD',data),
                                         doc='''''')
    sf_chan_hist = property(fget=lambda self: self._read_bf('sf_chan_hist'),
                                         fset=lambda self, data: self._error('Write access not allowed to field sf_chan_hist'),
                                         doc='''Reports the channel responsible for a sequence fault. If multiple channels fault, the lowest channel is reported. (3 bits, Read Only)
                                                 Preset 'NONE': 0
                                                 Preset 'V1': 1
                                                 Preset 'V2': 2
                                                 Preset 'V3': 3
                                                 Preset 'V4': 4
                                                 Preset 'V5': 5
                                                 Preset 'V6': 6
                                                 Preset 'UNDEFINED': 7''')
    sv_fault_status = property(fget=lambda self: self._read_bf('sv_fault_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field sv_fault_status'),
                                         doc='''Indicates the occurrence of a Supervisor fault. (1 bit, Read Only)
                                                 Preset 'SUPERVISOR_FAULT_HAS_NOT_OCCURRED': 0
                                                 Preset 'SUPERVISOR_FAULT_HAS_OCCURRED': 1''')
    v6_ov_sv_fault = property(fget=lambda self: self._read_bf('v6_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v6_ov_sv_fault'),
                                         doc='''V6 OV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v6_uv_sv_fault = property(fget=lambda self: self._read_bf('v6_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v6_uv_sv_fault'),
                                         doc='''V6 UV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v5_ov_sv_fault = property(fget=lambda self: self._read_bf('v5_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v5_ov_sv_fault'),
                                         doc='''V5 OV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v5_uv_sv_fault = property(fget=lambda self: self._read_bf('v5_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v5_uv_sv_fault'),
                                         doc='''V5 UV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v4_ov_sv_fault = property(fget=lambda self: self._read_bf('v4_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v4_ov_sv_fault'),
                                         doc='''V4 OV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v4_uv_sv_fault = property(fget=lambda self: self._read_bf('v4_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v4_uv_sv_fault'),
                                         doc='''V4 UV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v3_ov_sv_fault = property(fget=lambda self: self._read_bf('v3_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v3_ov_sv_fault'),
                                         doc='''V3 OV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v3_uv_sv_fault = property(fget=lambda self: self._read_bf('v3_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v3_uv_sv_fault'),
                                         doc='''V3 UV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v2_ov_sv_fault = property(fget=lambda self: self._read_bf('v2_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v2_ov_sv_fault'),
                                         doc='''V2 OV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v2_uv_sv_fault = property(fget=lambda self: self._read_bf('v2_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v2_uv_sv_fault'),
                                         doc='''V2 UV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v1_ov_sv_fault = property(fget=lambda self: self._read_bf('v1_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v1_ov_sv_fault'),
                                         doc='''V1 OV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    v1_uv_sv_fault = property(fget=lambda self: self._read_bf('v1_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v1_uv_sv_fault'),
                                         doc='''V1 UV Supervisor Fault history. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    MONITOR_STATUS_HISTORY_CMD = property(fget=lambda self: self._read_bf('MONITOR_STATUS_HISTORY_CMD'),
                                         fset=lambda self, data: self._error('Write access not allowed to register MONITOR_STATUS_HISTORY_CMD'),
                                         doc='''''')
    CLEAR_ALERTB = property(fget=lambda self: self._read_bf('CLEAR_ALERTB'),
                                         fset=lambda self, data: self._error('Write access not allowed to field CLEAR_ALERTB'),
                                         doc='''Release the ALERTB pin. (16 bits, Read Only)''')
    sf_channel = property(fget=lambda self: self._read_bf('sf_channel'),
                                         fset=lambda self, data: self._error('Write access not allowed to field sf_channel'),
                                         doc='''Reports the channel responsible for a sequence fault. If multiple channels fault, the lowest channel is reported. (3 bits, Read Only)
                                                 Preset 'NONE': 0
                                                 Preset 'V1': 1
                                                 Preset 'V2': 2
                                                 Preset 'V3': 3
                                                 Preset 'V4': 4
                                                 Preset 'V5': 5
                                                 Preset 'V6': 6
                                                 Preset 'UNDEFINED': 7''')
    mb_state = property(fget=lambda self: self._read_bf('mb_state'),
                                         fset=lambda self, data: self._error('Write access not allowed to field mb_state'),
                                         doc='''Monitor Backup Status. (1 bit, Read Only)
                                                 Preset 'EMPTY': 0
                                                 Preset 'WRITTEN': 1''')
    local_seq_status = property(fget=lambda self: self._read_bf('local_seq_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field local_seq_status'),
                                         doc='''Addressed Device Sequencing Status. (2 bits, Read Only)
                                                 Preset 'SEQUENCE_DOWN_DONE': 0
                                                 Preset 'SEQUENCE_UP_IN_PROGRESS': 1
                                                 Preset 'SEQUENCE_DOWN_IN_PROGRESS': 2
                                                 Preset 'SEQUENCE_UP_DONE': 3''')
    group_seq_status = property(fget=lambda self: self._read_bf('group_seq_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field group_seq_status'),
                                         doc='''Group Sequencing Status. (2 bits, Read Only)
                                                 Preset 'SEQUENCE_DOWN_DONE': 0
                                                 Preset 'SEQUENCE_UP_IN_PROGRESS': 1
                                                 Preset 'SEQUENCE_DOWN_IN_PROGRESS': 2
                                                 Preset 'SEQUENCE_UP_DONE': 3''')
    seq_up_fault = property(fget=lambda self: self._read_bf('seq_up_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field seq_up_fault'),
                                         doc='''Maximum Turn_on time Fault. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    seq_down_fault = property(fget=lambda self: self._read_bf('seq_down_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field seq_down_fault'),
                                         doc='''Maximum Turn_off time Fault. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    ov_fault = property(fget=lambda self: self._read_bf('ov_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field ov_fault'),
                                         doc='''Supervisor fault by mapped OV channel. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    uv_fault = property(fget=lambda self: self._read_bf('uv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field uv_fault'),
                                         doc='''Supervisor fault by mapped UV channel. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    sv_fault = property(fget=lambda self: self._read_bf('sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field sv_fault'),
                                         doc='''Supervisor Fault Status. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    discharge = property(fget=lambda self: self._read_bf('discharge'),
                                         fset=lambda self, data: self._error('Write access not allowed to field discharge'),
                                         doc='''Sequenced channel group discharge status. (1 bit, Read Only)
                                                 Preset 'ONE_OR_MORE_SEQUENCED_CHANNELS_NOT_DISCHARGED': 0
                                                 Preset 'ALL_SEQUENCED_CHANNELS_ARE_BELOW_DISCHARGE_THRESHOLDS': 1''')
    seq_control_fault = property(fget=lambda self: self._read_bf('seq_control_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field seq_control_fault'),
                                         doc='''Control Fault Status. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT': 1''')
    other_fault = property(fget=lambda self: self._read_bf('other_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field other_fault'),
                                         doc='''External or SHARE_CLK fault status. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    STATUS_INFORMATION_CMD = property(fget=lambda self: self._read_bf('STATUS_INFORMATION_CMD'),
                                         fset=lambda self, data: self._error('Write access not allowed to register STATUS_INFORMATION_CMD'),
                                         doc='''''')
    bp_enable = property(fget=lambda self: self._read_bf('bp_enable'),
                                         fset=lambda self,data: self._write_bf('bp_enable',data),
                                         doc='''Break Point Enable. (1 bit, R/W)
                                                 Preset 'BREAK_POINT_NOT_ENABLED': 0
                                                 Preset 'BREAK_POINT_ENABLED': 1''')
    bp_value = property(fget=lambda self: self._read_bf('bp_value'),
                                         fset=lambda self,data: self._write_bf('bp_value',data),
                                         doc='''Break Point Sequence Position Value. (10 bits, R/W)''')
    BREAK_POINT_CMD = property(fget=lambda self: self._read_bf('BREAK_POINT_CMD'),
                                         fset=lambda self,data: self._write_bf('BREAK_POINT_CMD',data),
                                         doc='''''')
    sp_bp_test = property(fget=lambda self: self._read_bf('sp_bp_test'),
                                         fset=lambda self, data: self._error('Write access not allowed to field sp_bp_test'),
                                         doc='''Compare sp_count with bp_value. (1 bit, Read Only)
                                                 Preset 'NOT_EQUAL': 0
                                                 Preset 'EQUAL': 1''')
    sp_count = property(fget=lambda self: self._read_bf('sp_count'),
                                         fset=lambda self, data: self._error('Write access not allowed to field sp_count'),
                                         doc='''Sequence Position Count. (10 bits, Read Only)''')
    SEQ_POSITION_COUNT_CMD = property(fget=lambda self: self._read_bf('SEQ_POSITION_COUNT_CMD'),
                                         fset=lambda self, data: self._error('Write access not allowed to register SEQ_POSITION_COUNT_CMD'),
                                         doc='''''')
    bu_sf_chan = property(fget=lambda self: self._read_bf('bu_sf_chan'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_sf_chan'),
                                         doc='''Reports the channel responsible for a sequence fault. If multiple channels fault, the lowest channel is reported. (3 bits, Read Only)
                                                 Preset 'NONE': 0
                                                 Preset 'V1': 1
                                                 Preset 'V2': 2
                                                 Preset 'V3': 3
                                                 Preset 'V4': 4
                                                 Preset 'V5': 5
                                                 Preset 'V6': 6
                                                 Preset 'UNDEFINED': 7''')
    bu_svf_state = property(fget=lambda self: self._read_bf('bu_svf_state'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_svf_state'),
                                         doc='''Supervisor Fault status backup. (1 bit, Read Only)
                                                 Preset 'SUPERVISOR_FAULT_HAS_NOT_OCCURRED': 0
                                                 Preset 'SUPERVISOR_FAULT_HAS_OCCURRED': 1''')
    bu_v6_ov_sv_fault = property(fget=lambda self: self._read_bf('bu_v6_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v6_ov_sv_fault'),
                                         doc='''V6 OV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v6_uv_sv_fault = property(fget=lambda self: self._read_bf('bu_v6_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v6_uv_sv_fault'),
                                         doc='''V6 UV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v5_ov_sv_fault = property(fget=lambda self: self._read_bf('bu_v5_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v5_ov_sv_fault'),
                                         doc='''V5 OV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v5_uv_sv_fault = property(fget=lambda self: self._read_bf('bu_v5_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v5_uv_sv_fault'),
                                         doc='''V5 UV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v4_ov_sv_fault = property(fget=lambda self: self._read_bf('bu_v4_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v4_ov_sv_fault'),
                                         doc='''V4 OV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v4_uv_sv_fault = property(fget=lambda self: self._read_bf('bu_v4_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v4_uv_sv_fault'),
                                         doc='''V4 UV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v3_ov_sv_fault = property(fget=lambda self: self._read_bf('bu_v3_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v3_ov_sv_fault'),
                                         doc='''V3 OV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v3_uv_sv_fault = property(fget=lambda self: self._read_bf('bu_v3_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v3_uv_sv_fault'),
                                         doc='''V3 UV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v2_ov_sv_fault = property(fget=lambda self: self._read_bf('bu_v2_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v2_ov_sv_fault'),
                                         doc='''V2 OV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v2_uv_sv_fault = property(fget=lambda self: self._read_bf('bu_v2_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v2_uv_sv_fault'),
                                         doc='''V2 UV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v1_ov_sv_fault = property(fget=lambda self: self._read_bf('bu_v1_ov_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v1_ov_sv_fault'),
                                         doc='''V1 OV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    bu_v1_uv_sv_fault = property(fget=lambda self: self._read_bf('bu_v1_uv_sv_fault'),
                                         fset=lambda self, data: self._error('Write access not allowed to field bu_v1_uv_sv_fault'),
                                         doc='''V1 UV Supervisor Fault backup. (1 bit, Read Only)
                                                 Preset 'NO_FAULT': 0
                                                 Preset 'FAULT_OCCURRED': 1''')
    MONITOR_BACKUP_CMD = property(fget=lambda self: self._read_bf('MONITOR_BACKUP_CMD'),
                                         fset=lambda self, data: self._error('Write access not allowed to register MONITOR_BACKUP_CMD'),
                                         doc='''''')
    margin_status = property(fget=lambda self: self._read_bf('margin_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field margin_status'),
                                         doc='''Logical representation of external margb input and/or i2c margin bit. (1 bit, Read Only)
                                                 Preset 'MARGIN_FUNCTION_INACTIVE': 0
                                                 Preset 'MARGIN_FUNCTION_ACTIVE': 1''')
    rstb_status = property(fget=lambda self: self._read_bf('rstb_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field rstb_status'),
                                         doc='''Status of the RSTB pin. (1 bit, Read Only)
                                                 Preset 'RSTB_IS_HIGH': 0
                                                 Preset 'RSTB_IS_LOW': 1''')
    v6_ov_status = property(fget=lambda self: self._read_bf('v6_ov_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v6_ov_status'),
                                         doc='''V6 OV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'HIGH_LIMIT_VIOLATION': 1''')
    v6_uv_status = property(fget=lambda self: self._read_bf('v6_uv_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v6_uv_status'),
                                         doc='''V6 UV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'LOW_LIMIT_VIOLATION': 1''')
    v5_ov_status = property(fget=lambda self: self._read_bf('v5_ov_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v5_ov_status'),
                                         doc='''V5 OV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'HIGH_LIMIT_VIOLATION': 1''')
    v5_uv_status = property(fget=lambda self: self._read_bf('v5_uv_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v5_uv_status'),
                                         doc='''V5 UV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'LOW_LIMIT_VIOLATION': 1''')
    v4_ov_status = property(fget=lambda self: self._read_bf('v4_ov_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v4_ov_status'),
                                         doc='''V4 OV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'HIGH_LIMIT_VIOLATION': 1''')
    v4_uv_status = property(fget=lambda self: self._read_bf('v4_uv_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v4_uv_status'),
                                         doc='''V4 UV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'LOW_LIMIT_VIOLATION': 1''')
    v3_ov_status = property(fget=lambda self: self._read_bf('v3_ov_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v3_ov_status'),
                                         doc='''V3 OV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'HIGH_LIMIT_VIOLATION': 1''')
    v3_uv_status = property(fget=lambda self: self._read_bf('v3_uv_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v3_uv_status'),
                                         doc='''V3 UV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'LOW_LIMIT_VIOLATION': 1''')
    v2_ov_status = property(fget=lambda self: self._read_bf('v2_ov_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v2_ov_status'),
                                         doc='''V2 OV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'HIGH_LIMIT_VIOLATION': 1''')
    v2_uv_status = property(fget=lambda self: self._read_bf('v2_uv_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v2_uv_status'),
                                         doc='''V2 UV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'LOW_LIMIT_VIOLATION': 1''')
    v1_ov_status = property(fget=lambda self: self._read_bf('v1_ov_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v1_ov_status'),
                                         doc='''V1 OV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'HIGH_LIMIT_VIOLATION': 1''')
    v1_uv_status = property(fget=lambda self: self._read_bf('v1_uv_status'),
                                         fset=lambda self, data: self._error('Write access not allowed to field v1_uv_status'),
                                         doc='''V1 UV comparator live status. (1 bit, Read Only)
                                                 Preset 'NO_VIOLATION': 0
                                                 Preset 'LOW_LIMIT_VIOLATION': 1''')
    MONITOR_STATUS_CMD = property(fget=lambda self: self._read_bf('MONITOR_STATUS_CMD'),
                                         fset=lambda self, data: self._error('Write access not allowed to register MONITOR_STATUS_CMD'),
                                         doc='''''')
    DEVICE_ID = property(fget=lambda self: self._read_bf('DEVICE_ID'),
                                         fset=lambda self, data: self._error('Write access not allowed to field DEVICE_ID'),
                                         doc='''Contents of DEVICE_ID = 0x2937 (16 bits, Read Only)''')

    _constants = {
        }

    _xml_formats = {
        'Delay_time_to_Integer': {'points': [('0', '0'), ('1', '80E-6')], 'description': '''convert an Enable Delay to 13-bit integer''', 'signed': False},
        'Volts_to_TE_HR': {'points': [('0', '0.9'), ('1', '0.9 + 0.02')], 'description': '''convert a voltage value to Threshold Encoding for High Range''', 'signed': False},
        'Volts_to_TE_LR': {'points': [('0', '0.45'), ('1', '0.45 + 0.01')], 'description': '''convert a voltage value to Threshold Encoding for Low Range''', 'signed': False},
        'Volts_to_TE_AR': {'points': [('0', '0.18'), ('1', '0.18 + 0.004')], 'description': '''convert a voltage value to Threshold Encoding for Adjustable Range''', 'signed': False},
        }

    #Caution! This register map, including active_format settings is shared across all instances of the class.
    _register_map = {
        'device_key': {'command_code':0x00, 'size':14, 'offset':2, 'read_presets_enabled':True, 'presets':[['DEFAULT',3754],], 'allowed_formats':[],'active_format':'None'},
        'hw_lock_bit': {'command_code':0x00, 'size':1, 'offset':1, 'read_presets_enabled':True, 'presets':[['UNLOCKED',0],['LOCKED',1],], 'allowed_formats':[],'active_format':'None'},
        'sw_lock_bit': {'command_code':0x00, 'size':1, 'offset':0, 'read_presets_enabled':True, 'presets':[['UNLOCKED',0],['LOCKED',1],], 'allowed_formats':[],'active_format':'None'},
        'WRITE_PROTECTION_CMD': {'command_code':0x00, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SPECIAL_LOT': {'command_code':0x01, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'on_state': {'command_code':0x02, 'size':1, 'offset':7, 'read_presets_enabled':True, 'presets':[['INTERNAL_ON_LOW',0],['INTERNAL_ON_HIGH',1],], 'allowed_formats':[],'active_format':'None'},
        'i2c_margin': {'command_code':0x02, 'size':1, 'offset':6, 'read_presets_enabled':True, 'presets':[['RSTB_OPERATES_NORMALLY',0],['RSTB_PULLS_HIGH',1],], 'allowed_formats':[],'active_format':'None'},
        'discharge_start': {'command_code':0x02, 'size':1, 'offset':5, 'read_presets_enabled':True, 'presets':[['DISCHARGE_NOT_REQUIRED',0],['DISCHARGE_REQUIRED',1],], 'allowed_formats':[],'active_format':'None'},
        'i2c_on_off': {'command_code':0x02, 'size':1, 'offset':4, 'read_presets_enabled':True, 'presets':[['SEQUENCE_DOWN',0],['SEQUENCE_UP',1],], 'allowed_formats':[],'active_format':'None'},
        'i2c_on_off_mask': {'command_code':0x02, 'size':1, 'offset':3, 'read_presets_enabled':True, 'presets':[['I2C_IGNORE',0],['I2C_LISTEN',1],], 'allowed_formats':[],'active_format':'None'},
        'on_input_mask': {'command_code':0x02, 'size':1, 'offset':2, 'read_presets_enabled':True, 'presets':[['ON_PIN_IGNORE',0],['ON_PIN_LISTEN',1],], 'allowed_formats':[],'active_format':'None'},
        'on_polarity': {'command_code':0x02, 'size':1, 'offset':1, 'read_presets_enabled':True, 'presets':[['ACTIVE_HIGH',0],['ACTIVE_LOW',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_down_qual': {'command_code':0x02, 'size':1, 'offset':0, 'read_presets_enabled':True, 'presets':[['DISCHARGE_BASED',0],['TIME_BASED',1],], 'allowed_formats':[],'active_format':'None'},
        'ON_OFF_CONTROL_CMD': {'command_code':0x02, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'v6_range': {'command_code':0x03, 'size':2, 'offset':10, 'read_presets_enabled':True, 'presets':[['HIGH_RANGE',0],['LOW_RANGE',1],['POSITIVE_ADJUSTABLE',2],['NEGATIVE_ADJUSTABLE',3],], 'allowed_formats':[],'active_format':'None'},
        'v5_range': {'command_code':0x03, 'size':2, 'offset':8, 'read_presets_enabled':True, 'presets':[['HIGH_RANGE',0],['LOW_RANGE',1],['POSITIVE_ADJUSTABLE',2],['NEGATIVE_ADJUSTABLE',3],], 'allowed_formats':[],'active_format':'None'},
        'v4_range': {'command_code':0x03, 'size':2, 'offset':6, 'read_presets_enabled':True, 'presets':[['HIGH_RANGE',0],['LOW_RANGE',1],['POSITIVE_ADJUSTABLE',2],['NEGATIVE_ADJUSTABLE',3],], 'allowed_formats':[],'active_format':'None'},
        'v3_range': {'command_code':0x03, 'size':2, 'offset':4, 'read_presets_enabled':True, 'presets':[['HIGH_RANGE',0],['LOW_RANGE',1],['POSITIVE_ADJUSTABLE',2],['NEGATIVE_ADJUSTABLE',3],], 'allowed_formats':[],'active_format':'None'},
        'v2_range': {'command_code':0x03, 'size':2, 'offset':2, 'read_presets_enabled':True, 'presets':[['HIGH_RANGE',0],['LOW_RANGE',1],['POSITIVE_ADJUSTABLE',2],['NEGATIVE_ADJUSTABLE',3],], 'allowed_formats':[],'active_format':'None'},
        'v1_range': {'command_code':0x03, 'size':2, 'offset':0, 'read_presets_enabled':True, 'presets':[['HIGH_RANGE',0],['LOW_RANGE',1],['POSITIVE_ADJUSTABLE',2],['NEGATIVE_ADJUSTABLE',3],], 'allowed_formats':[],'active_format':'None'},
        'V_RANGE_CMD': {'command_code':0x03, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ov_threshold_1': {'command_code':0x04, 'size':8, 'offset':8, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_LR', 'Volts_to_TE_HR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_LR'},
        'uv_threshold_1': {'command_code':0x04, 'size':8, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_LR', 'Volts_to_TE_HR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_LR'},
        'V_THRESHOLD_1_CMD': {'command_code':0x04, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ov_threshold_2': {'command_code':0x05, 'size':8, 'offset':8, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_LR', 'Volts_to_TE_HR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_LR'},
        'uv_threshold_2': {'command_code':0x05, 'size':8, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_LR', 'Volts_to_TE_HR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_LR'},
        'V_THRESHOLD_2_CMD': {'command_code':0x05, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ov_threshold_3': {'command_code':0x06, 'size':8, 'offset':8, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_LR', 'Volts_to_TE_HR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_LR'},
        'uv_threshold_3': {'command_code':0x06, 'size':8, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_LR', 'Volts_to_TE_HR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_LR'},
        'V_THRESHOLD_3_CMD': {'command_code':0x06, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ov_threshold_4': {'command_code':0x07, 'size':8, 'offset':8, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_LR', 'Volts_to_TE_HR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_LR'},
        'uv_threshold_4': {'command_code':0x07, 'size':8, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_LR', 'Volts_to_TE_HR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_LR'},
        'V_THRESHOLD_4_CMD': {'command_code':0x07, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ov_threshold_5': {'command_code':0x08, 'size':8, 'offset':8, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_HR', 'Volts_to_TE_LR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_HR'},
        'uv_threshold_5': {'command_code':0x08, 'size':8, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_HR', 'Volts_to_TE_LR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_HR'},
        'V_THRESHOLD_5_CMD': {'command_code':0x08, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ov_threshold_6': {'command_code':0x09, 'size':8, 'offset':8, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_HR', 'Volts_to_TE_LR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_HR'},
        'uv_threshold_6': {'command_code':0x09, 'size':8, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Volts_to_TE_HR', 'Volts_to_TE_LR', 'Volts_to_TE_AR'],'active_format':'Volts_to_TE_HR'},
        'V_THRESHOLD_6_CMD': {'command_code':0x09, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ton_max_1': {'command_code':0x0a, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['160us',1],['640us',2],['2_6ms',3],['10_2ms',4],['41ms',5],['164ms',6],['655ms',7],], 'allowed_formats':[],'active_format':'None'},
        'ton_delay_1': {'command_code':0x0a, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TON_TIMERS_1_CMD': {'command_code':0x0a, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ton_max_2': {'command_code':0x0b, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['160us',1],['640us',2],['2_6ms',3],['10_2ms',4],['41ms',5],['164ms',6],['655ms',7],], 'allowed_formats':[],'active_format':'None'},
        'ton_delay_2': {'command_code':0x0b, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TON_TIMERS_2_CMD': {'command_code':0x0b, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ton_max_3': {'command_code':0x0c, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['160us',1],['640us',2],['2_6ms',3],['10_2ms',4],['41ms',5],['164ms',6],['655ms',7],], 'allowed_formats':[],'active_format':'None'},
        'ton_delay_3': {'command_code':0x0c, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TON_TIMERS_3_CMD': {'command_code':0x0c, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ton_max_4': {'command_code':0x0d, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['160us',1],['640us',2],['2_6ms',3],['10_2ms',4],['41ms',5],['164ms',6],['655ms',7],], 'allowed_formats':[],'active_format':'None'},
        'ton_delay_4': {'command_code':0x0d, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TON_TIMERS_4_CMD': {'command_code':0x0d, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ton_max_5': {'command_code':0x0e, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['160us',1],['640us',2],['2_6ms',3],['10_2ms',4],['41ms',5],['164ms',6],['655ms',7],], 'allowed_formats':[],'active_format':'None'},
        'ton_delay_5': {'command_code':0x0e, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TON_TIMERS_5_CMD': {'command_code':0x0e, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'ton_max_6': {'command_code':0x0f, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['160us',1],['640us',2],['2_6ms',3],['10_2ms',4],['41ms',5],['164ms',6],['655ms',7],], 'allowed_formats':[],'active_format':'None'},
        'ton_delay_6': {'command_code':0x0f, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TON_TIMERS_6_CMD': {'command_code':0x0f, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'toff_max_1': {'command_code':0x10, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['2_6ms',1],['10_2ms',2],['41ms',3],['164ms',4],['655ms',5],['2_6s',6],['10_5s',7],], 'allowed_formats':[],'active_format':'None'},
        'toff_delay_1': {'command_code':0x10, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TOFF_TIMERS_1_CMD': {'command_code':0x10, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'toff_max_2': {'command_code':0x11, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['2_6ms',1],['10_2ms',2],['41ms',3],['164ms',4],['655ms',5],['2_6s',6],['10_5s',7],], 'allowed_formats':[],'active_format':'None'},
        'toff_delay_2': {'command_code':0x11, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TOFF_TIMERS_2_CMD': {'command_code':0x11, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'toff_max_3': {'command_code':0x12, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['2_6ms',1],['10_2ms',2],['41ms',3],['164ms',4],['655ms',5],['2_6s',6],['10_5s',7],], 'allowed_formats':[],'active_format':'None'},
        'toff_delay_3': {'command_code':0x12, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TOFF_TIMERS_3_CMD': {'command_code':0x12, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'toff_max_4': {'command_code':0x13, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['2_6ms',1],['10_2ms',2],['41ms',3],['164ms',4],['655ms',5],['2_6s',6],['10_5s',7],], 'allowed_formats':[],'active_format':'None'},
        'toff_delay_4': {'command_code':0x13, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TOFF_TIMERS_4_CMD': {'command_code':0x13, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'toff_max_5': {'command_code':0x14, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['2_6ms',1],['10_2ms',2],['41ms',3],['164ms',4],['655ms',5],['2_6s',6],['10_5s',7],], 'allowed_formats':[],'active_format':'None'},
        'toff_delay_5': {'command_code':0x14, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TOFF_TIMERS_5_CMD': {'command_code':0x14, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'toff_max_6': {'command_code':0x15, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['INFINITY',0],['2_6ms',1],['10_2ms',2],['41ms',3],['164ms',4],['655ms',5],['2_6s',6],['10_5s',7],], 'allowed_formats':[],'active_format':'None'},
        'toff_delay_6': {'command_code':0x15, 'size':13, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':['Delay_time_to_Integer'],'active_format':'Delay_time_to_Integer'},
        'TOFF_TIMERS_6_CMD': {'command_code':0x15, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'async_on_off_1': {'command_code':0x16, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['HOLD_EN1_LOW',0],['RELEASE_EN1',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_up_position_1': {'command_code':0x16, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_UP_POSITION_1_CMD': {'command_code':0x16, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'async_on_off_2': {'command_code':0x17, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['HOLD_EN2_LOW',0],['RELEASE_EN2',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_up_position_2': {'command_code':0x17, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_UP_POSITION_2_CMD': {'command_code':0x17, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'async_on_off_3': {'command_code':0x18, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['HOLD_EN3_LOW',0],['RELEASE_EN3',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_up_position_3': {'command_code':0x18, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_UP_POSITION_3_CMD': {'command_code':0x18, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'async_on_off_4': {'command_code':0x19, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['HOLD_EN4_LOW',0],['RELEASE_EN4',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_up_position_4': {'command_code':0x19, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_UP_POSITION_4_CMD': {'command_code':0x19, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'async_on_off_5': {'command_code':0x1a, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['HOLD_EN5_LOW',0],['RELEASE_EN5',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_up_position_5': {'command_code':0x1a, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_UP_POSITION_5_CMD': {'command_code':0x1a, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'async_on_off_6': {'command_code':0x1b, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['HOLD_EN6_LOW',0],['RELEASE_EN6',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_up_position_6': {'command_code':0x1b, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_UP_POSITION_6_CMD': {'command_code':0x1b, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'active_pull_down_1': {'command_code':0x1c, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['PULL_DOWN_DISABLED',0],['PULL_DOWN_ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_down_position_1': {'command_code':0x1c, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_DOWN_POSITION_1_CMD': {'command_code':0x1c, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'active_pull_down_2': {'command_code':0x1d, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['PULL_DOWN_DISABLED',0],['PULL_DOWN_ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_down_position_2': {'command_code':0x1d, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_DOWN_POSITION_2_CMD': {'command_code':0x1d, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'active_pull_down_3': {'command_code':0x1e, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['PULL_DOWN_DISABLED',0],['PULL_DOWN_ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_down_position_3': {'command_code':0x1e, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_DOWN_POSITION_3_CMD': {'command_code':0x1e, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'active_pull_down_4': {'command_code':0x1f, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['PULL_DOWN_DISABLED',0],['PULL_DOWN_ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_down_position_4': {'command_code':0x1f, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_DOWN_POSITION_4_CMD': {'command_code':0x1f, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'active_pull_down_5': {'command_code':0x20, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['PULL_DOWN_DISABLED',0],['PULL_DOWN_ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_down_position_5': {'command_code':0x20, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_DOWN_POSITION_5_CMD': {'command_code':0x20, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'active_pull_down_6': {'command_code':0x21, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['PULL_DOWN_DISABLED',0],['PULL_DOWN_ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_down_position_6': {'command_code':0x21, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_DOWN_POSITION_6_CMD': {'command_code':0x21, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'rstb_delay': {'command_code':0x22, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['0_05ms',0],['1_6ms',1],['6_4ms',2],['26ms',3],['51ms',4],['200ms',5],['410ms',6],['1640ms',7],], 'allowed_formats':[],'active_format':'None'},
        'sv_fault_enable': {'command_code':0x22, 'size':1, 'offset':12, 'read_presets_enabled':True, 'presets':[['SUPERVISOR_FAULT_DOES_NOT_PULL_FAULTB',0],['SUPERVISOR_FAULT_PULLS_FAULTB',1],], 'allowed_formats':[],'active_format':'None'},
        'v6_ov_enable': {'command_code':0x22, 'size':1, 'offset':11, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v5_ov_enable': {'command_code':0x22, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v4_ov_enable': {'command_code':0x22, 'size':1, 'offset':9, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v3_ov_enable': {'command_code':0x22, 'size':1, 'offset':8, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v2_ov_enable': {'command_code':0x22, 'size':1, 'offset':7, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v1_ov_enable': {'command_code':0x22, 'size':1, 'offset':6, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v6_uv_enable': {'command_code':0x22, 'size':1, 'offset':5, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v5_uv_enable': {'command_code':0x22, 'size':1, 'offset':4, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v4_uv_enable': {'command_code':0x22, 'size':1, 'offset':3, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v3_uv_enable': {'command_code':0x22, 'size':1, 'offset':2, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v2_uv_enable': {'command_code':0x22, 'size':1, 'offset':1, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'v1_uv_enable': {'command_code':0x22, 'size':1, 'offset':0, 'read_presets_enabled':True, 'presets':[['NOT_ENABLED',0],['ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'RSTB_CONFIG_CMD': {'command_code':0x22, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'retry_count': {'command_code':0x23, 'size':3, 'offset':11, 'read_presets_enabled':True, 'presets':[['UNDEFINED',7],], 'allowed_formats':[],'active_format':'None'},
        'faultb_state': {'command_code':0x23, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['HIGH',1],['LOW',0],], 'allowed_formats':[],'active_format':'None'},
        'faultb_mask': {'command_code':0x23, 'size':1, 'offset':9, 'read_presets_enabled':True, 'presets':[['IGNORE_EXTERNAL_FAULTB_PULLDOWN',0],['RESPOND_TO_EXTERNAL_FAULTB_PULLDOWN',1],], 'allowed_formats':[],'active_format':'None'},
        'freeze': {'command_code':0x23, 'size':1, 'offset':8, 'read_presets_enabled':True, 'presets':[['DO_NOT_FREEZE_ON_FAULT',0],['FREEZE_DEVICE_STATE_ON_FAULT',1],], 'allowed_formats':[],'active_format':'None'},
        'retry_delay': {'command_code':0x23, 'size':3, 'offset':5, 'read_presets_enabled':True, 'presets':[['0_05ms',0],['200ms',1],['410ms',2],['820ms',3],['1_64s',4],['3_28s',5],['6_55s',6],['13_1s',7],], 'allowed_formats':[],'active_format':'None'},
        'fault_response_action': {'command_code':0x23, 'size':2, 'offset':3, 'read_presets_enabled':True, 'presets':[['CONTINUE_OPERATION',0],['DISCHARGED_RETRY',1],['DELAYED_RETRY',2],['DISCHARGE_AND_DELAY_RETRY',3],], 'allowed_formats':[],'active_format':'None'},
        'retry_number': {'command_code':0x23, 'size':3, 'offset':0, 'read_presets_enabled':True, 'presets':[['DO_NOT_ATTEMPT_TO_RETRY',0],['RETRY_1_TIMES',1],['RETRY_2_TIMES',2],['RETRY_3_TIMES',3],['RETRY_4_TIMES',4],['RETRY_5_TIMES',5],['RETRY_6_TIMES',6],['UNLIMITED_RETRIES',7],], 'allowed_formats':[],'active_format':'None'},
        'FAULT_RESPONSE_CMD': {'command_code':0x23, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'sf_chan_hist': {'command_code':0x26, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['NONE',0],['V1',1],['V2',2],['V3',3],['V4',4],['V5',5],['V6',6],['UNDEFINED',7],], 'allowed_formats':[],'active_format':'None'},
        'sv_fault_status': {'command_code':0x26, 'size':1, 'offset':12, 'read_presets_enabled':True, 'presets':[['SUPERVISOR_FAULT_HAS_NOT_OCCURRED',0],['SUPERVISOR_FAULT_HAS_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v6_ov_sv_fault': {'command_code':0x26, 'size':1, 'offset':11, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v6_uv_sv_fault': {'command_code':0x26, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v5_ov_sv_fault': {'command_code':0x26, 'size':1, 'offset':9, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v5_uv_sv_fault': {'command_code':0x26, 'size':1, 'offset':8, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v4_ov_sv_fault': {'command_code':0x26, 'size':1, 'offset':7, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v4_uv_sv_fault': {'command_code':0x26, 'size':1, 'offset':6, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v3_ov_sv_fault': {'command_code':0x26, 'size':1, 'offset':5, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v3_uv_sv_fault': {'command_code':0x26, 'size':1, 'offset':4, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v2_ov_sv_fault': {'command_code':0x26, 'size':1, 'offset':3, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v2_uv_sv_fault': {'command_code':0x26, 'size':1, 'offset':2, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v1_ov_sv_fault': {'command_code':0x26, 'size':1, 'offset':1, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'v1_uv_sv_fault': {'command_code':0x26, 'size':1, 'offset':0, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'MONITOR_STATUS_HISTORY_CMD': {'command_code':0x26, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'CLEAR_ALERTB': {'command_code':0x28, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'sf_channel': {'command_code':0x29, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['NONE',0],['V1',1],['V2',2],['V3',3],['V4',4],['V5',5],['V6',6],['UNDEFINED',7],], 'allowed_formats':[],'active_format':'None'},
        'mb_state': {'command_code':0x29, 'size':1, 'offset':12, 'read_presets_enabled':True, 'presets':[['EMPTY',0],['WRITTEN',1],], 'allowed_formats':[],'active_format':'None'},
        'local_seq_status': {'command_code':0x29, 'size':2, 'offset':10, 'read_presets_enabled':True, 'presets':[['SEQUENCE_DOWN_DONE',0],['SEQUENCE_UP_IN_PROGRESS',1],['SEQUENCE_DOWN_IN_PROGRESS',2],['SEQUENCE_UP_DONE',3],], 'allowed_formats':[],'active_format':'None'},
        'group_seq_status': {'command_code':0x29, 'size':2, 'offset':8, 'read_presets_enabled':True, 'presets':[['SEQUENCE_DOWN_DONE',0],['SEQUENCE_UP_IN_PROGRESS',1],['SEQUENCE_DOWN_IN_PROGRESS',2],['SEQUENCE_UP_DONE',3],], 'allowed_formats':[],'active_format':'None'},
        'seq_up_fault': {'command_code':0x29, 'size':1, 'offset':7, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_down_fault': {'command_code':0x29, 'size':1, 'offset':6, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'ov_fault': {'command_code':0x29, 'size':1, 'offset':5, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'uv_fault': {'command_code':0x29, 'size':1, 'offset':4, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'sv_fault': {'command_code':0x29, 'size':1, 'offset':3, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'discharge': {'command_code':0x29, 'size':1, 'offset':2, 'read_presets_enabled':True, 'presets':[['ONE_OR_MORE_SEQUENCED_CHANNELS_NOT_DISCHARGED',0],['ALL_SEQUENCED_CHANNELS_ARE_BELOW_DISCHARGE_THRESHOLDS',1],], 'allowed_formats':[],'active_format':'None'},
        'seq_control_fault': {'command_code':0x29, 'size':1, 'offset':1, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT',1],], 'allowed_formats':[],'active_format':'None'},
        'other_fault': {'command_code':0x29, 'size':1, 'offset':0, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'STATUS_INFORMATION_CMD': {'command_code':0x29, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'bp_enable': {'command_code':0x2a, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['BREAK_POINT_NOT_ENABLED',0],['BREAK_POINT_ENABLED',1],], 'allowed_formats':[],'active_format':'None'},
        'bp_value': {'command_code':0x2a, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'BREAK_POINT_CMD': {'command_code':0x2a, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'sp_bp_test': {'command_code':0x2b, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['NOT_EQUAL',0],['EQUAL',1],], 'allowed_formats':[],'active_format':'None'},
        'sp_count': {'command_code':0x2b, 'size':10, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'SEQ_POSITION_COUNT_CMD': {'command_code':0x2b, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'bu_sf_chan': {'command_code':0x2f, 'size':3, 'offset':13, 'read_presets_enabled':True, 'presets':[['NONE',0],['V1',1],['V2',2],['V3',3],['V4',4],['V5',5],['V6',6],['UNDEFINED',7],], 'allowed_formats':[],'active_format':'None'},
        'bu_svf_state': {'command_code':0x2f, 'size':1, 'offset':12, 'read_presets_enabled':True, 'presets':[['SUPERVISOR_FAULT_HAS_NOT_OCCURRED',0],['SUPERVISOR_FAULT_HAS_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v6_ov_sv_fault': {'command_code':0x2f, 'size':1, 'offset':11, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v6_uv_sv_fault': {'command_code':0x2f, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v5_ov_sv_fault': {'command_code':0x2f, 'size':1, 'offset':9, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v5_uv_sv_fault': {'command_code':0x2f, 'size':1, 'offset':8, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v4_ov_sv_fault': {'command_code':0x2f, 'size':1, 'offset':7, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v4_uv_sv_fault': {'command_code':0x2f, 'size':1, 'offset':6, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v3_ov_sv_fault': {'command_code':0x2f, 'size':1, 'offset':5, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v3_uv_sv_fault': {'command_code':0x2f, 'size':1, 'offset':4, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v2_ov_sv_fault': {'command_code':0x2f, 'size':1, 'offset':3, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v2_uv_sv_fault': {'command_code':0x2f, 'size':1, 'offset':2, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v1_ov_sv_fault': {'command_code':0x2f, 'size':1, 'offset':1, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'bu_v1_uv_sv_fault': {'command_code':0x2f, 'size':1, 'offset':0, 'read_presets_enabled':True, 'presets':[['NO_FAULT',0],['FAULT_OCCURRED',1],], 'allowed_formats':[],'active_format':'None'},
        'MONITOR_BACKUP_CMD': {'command_code':0x2f, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'margin_status': {'command_code':0x30, 'size':1, 'offset':13, 'read_presets_enabled':True, 'presets':[['MARGIN_FUNCTION_INACTIVE',0],['MARGIN_FUNCTION_ACTIVE',1],], 'allowed_formats':[],'active_format':'None'},
        'rstb_status': {'command_code':0x30, 'size':1, 'offset':12, 'read_presets_enabled':True, 'presets':[['RSTB_IS_HIGH',0],['RSTB_IS_LOW',1],], 'allowed_formats':[],'active_format':'None'},
        'v6_ov_status': {'command_code':0x30, 'size':1, 'offset':11, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['HIGH_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v6_uv_status': {'command_code':0x30, 'size':1, 'offset':10, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['LOW_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v5_ov_status': {'command_code':0x30, 'size':1, 'offset':9, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['HIGH_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v5_uv_status': {'command_code':0x30, 'size':1, 'offset':8, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['LOW_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v4_ov_status': {'command_code':0x30, 'size':1, 'offset':7, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['HIGH_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v4_uv_status': {'command_code':0x30, 'size':1, 'offset':6, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['LOW_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v3_ov_status': {'command_code':0x30, 'size':1, 'offset':5, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['HIGH_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v3_uv_status': {'command_code':0x30, 'size':1, 'offset':4, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['LOW_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v2_ov_status': {'command_code':0x30, 'size':1, 'offset':3, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['HIGH_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v2_uv_status': {'command_code':0x30, 'size':1, 'offset':2, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['LOW_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v1_ov_status': {'command_code':0x30, 'size':1, 'offset':1, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['HIGH_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'v1_uv_status': {'command_code':0x30, 'size':1, 'offset':0, 'read_presets_enabled':True, 'presets':[['NO_VIOLATION',0],['LOW_LIMIT_VIOLATION',1],], 'allowed_formats':[],'active_format':'None'},
        'MONITOR_STATUS_CMD': {'command_code':0x30, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
        'DEVICE_ID': {'command_code':0x31, 'size':16, 'offset':0, 'read_presets_enabled':True, 'presets':[], 'allowed_formats':[],'active_format':'None'},
    }

    _formatters = {}

class LTC2937_APIException(Exception):
    '''Bad data passed to instance of LTC2937 will raise LTC2937_APIException'''
    pass

import serial, time
class ltc_dc590b_interface(object):
    '''Python Communication Interface for DC590B / Linduino.
    Provides the following SMBus protocols with optional PEC (Packet Error Checking) CRC:
        ReadWord
        WriteWord
        ReadByte
        WriteByte
        ReceiveByte
        SendByte
        AlertResponse
    Requires PySerial.'''
    def __init__(self, serial_port, PEC=False, **kwargs):
        '''Provide serial port of attached Linear Technology DC590 I²C interface board or DC2026B Linduno board programmed with DC590 emulator sketch.  kwargs will be passed to PySerial init unless serial_port is already configured.'''
        if isinstance(serial_port, serial.Serial):
            self.serial_port = serial_port
        else:
            if 'timeout' not in kwargs:
                kwargs['timeout'] = 1
            if 'baudrate' not in kwargs:
                kwargs['baudrate'] = 115200
            self.serial_port = serial.Serial(serial_port, **kwargs)
        if PEC:
            self.read_word = self._read_word_pec
            self.write_word = self._write_word_pec
            self.read_byte = self._read_byte_pec
            self.write_byte = self._write_byte_pec
            self.receive_byte = self._receive_byte_pec
            self.send_byte = self._send_byte_pec
            self.alert_response = self._alert_response_pec
        else:
            self.read_word = self._read_word_nopec
            self.write_word = self._write_word_nopec
            self.read_byte = self._read_byte_nopec
            self.write_byte = self._write_byte_nopec
            self.receive_byte = self._receive_byte_nopec
            self.send_byte = self._send_byte_nopec
            self.alert_response = self._alert_response_nopec
        self._serial_wait = 0.1
        time.sleep(2.5) #Linduino bootloader delay!
        self.serial_port.write('\n'*10) 
        time.sleep(2.5) #Linduino bootloader delay!
        self.serial_port.write('MI') #Switch to isolated I²C Mode
        self.serial_port.write('O') #Enable isolated power
        time.sleep(self._serial_wait)
        print('DC590B init response: {}'.format(self.serial_port.read(self.serial_port.inWaiting()))) #discard any responses
    def read_word(self, addr_7bit, command_code):
        '''SMBus Read Word Protocol.
        Packet Error Checking controlled by class init.
        Slave device address specified in 7-bit format.
        Returns 16-bit data from slave.'''
    def write_word(self, addr_7bit, command_code, data16):
        '''SMBus Write Word Protocol.
        Packet Error Checking controlled by class init.
        Slave device address specified in 7-bit format.
        Returns None.'''
    def read_byte(self, addr_7bit, command_code):
        '''SMBus Read Byte Protocol.
        Packet Error Checking controlled by class init.
        Slave device address specified in 7-bit format.
        Returns 8-bit data from slave.'''
    def write_byte(self, addr_7bit, command_code, data8):
        '''SMBus Write Byte Protocol.
        Packet Error Checking controlled by class init.
        Slave device address specified in 7-bit format.
        Returns None.'''
    def receive_byte(self, addr_7bit):
        '''SMBus Receive Byte Protocol.
        Packet Error Checking controlled by class init.
        Slave device address specified in 7-bit format.
        Returns 8-bit data from slave.'''
    def send_byte(self, addr_7bit, data8):
        '''SMBus Send Byte Protocol.
        Packet Error Checking controlled by class init.
        Slave device address specified in 7-bit format.
        Returns None.'''
    def alert_response(self):
        '''SMBus Send Byte Protocol.
        Packet Error Checking controlled by class init.
        Slave device address specified in 7-bit format.
        Returns None if no response to ARA, otherwise the responding device address placed in the 7 most significant bits of the byte.
        The returned LSB (R/W̅ bit) will be zero in this implementation (i.e. write address).'''
    def _hex_str(self, integer):
        '''return integer formatted correctly for transmission over DC590 serial link'''
        return hex(integer)[2:].rjust(2,"0").upper()
    def _read_addr(self,addr_7bit):
        '''compute 8-bit read address from 7-bit address'''
        return (addr_7bit << 1) + 1
    def _write_addr(self, addr_7bit):
        '''compute 8-bit write address from 7-bit address'''
        return addr_7bit << 1
    def _word(self, lowByte, highByte):
        '''Return 16-bit value from two SMBus bytes'''
        return ((highByte & 0xFF) << 8) + (lowByte & 0xFF)
    def _lowByte(self, data16):
        return data16 & 0xFF
    def _highByte(self, data16):
        return data16 >> 8
    def _pec(self,byteList):
        '''byteList is an ordered list of every byte in the transaction including address, command code (subaddr) and data'''
        crc = 0
        poly = 0x07 #x^8 + x^2 + x^1 + 1, discard x^8 term
        for byte in byteList:
            crc ^= byte
            for cycle in range(8):
                crc <<= 1
                if (crc & 0x100): #msb was set before left shift ( & 0x80)
                    #xor with crc if pre-shift msb was 1
                    crc ^= poly
        return int(crc & 0xFF)
    def _read_word_nopec(self, addr_7bit, command_code):
        '''SMBus Read Word Protocol without Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns 16-bit data from slave.'''
        byteList = [self._write_addr(addr_7bit), command_code, self._read_addr(addr_7bit)]
        write_str = 'sS{}S{}sS{}QRp'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(4)
        if (len(ret_str) != 4):
            raise Exception('Short response to DC590 read_word command, EEPROM Present?: %s' % ret_str)
        if ('N' in ret_str):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('DC590 read_word failed acknowledge: {} then {}'.format(ret_str, ret_extra))
        if ('X' in ret_str):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('DC590 EEPROM detection failed, communications disabled: {} then {}'.format(ret_str, ret_extra))
        if (self.serial_port.inWaiting()):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('Long response to DC590 read_word command: {} then {}'.format(ret_str, ret_extra))
        return self._word(lowByte=int(ret_str[0:2],16), highByte=int(ret_str[2:4],16))
    def _write_word_nopec(self, addr_7bit, command_code, data16):
        '''SMBus Write Word Protocol without Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns None.'''
        byteList = [self._write_addr(addr_7bit), command_code, self._lowByte(data16), self._highByte(data16)]
        write_str = 'sS{}S{}S{}S{}p'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(self.serial_port.inWaiting())
        if (len(ret_str) != 0):
            raise Exception('Response: {} from DC590 write_word command'.format(ret_str))
    def _read_byte_nopec(self, addr_7bit, command_code):
        '''SMBus Read Byte Protocol without Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns 8-bit data from slave.'''
        byteList = [self._write_addr(addr_7bit), command_code, self._read_addr(addr_7bit)]
        write_str = 'sS{}S{}sS{}Rp'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(2)
        if (len(ret_str) != 2):
            raise Exception('Short response to DC590 read_byte command, EEPROM Present?: %s' % ret_str)
        if ('N' in ret_str):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('DC590 read_byte failed acknowledge: {} then {}'.format(ret_str, ret_extra))
        if ('X' in ret_str):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('DC590 EEPROM detection failed, communications disabled: {} then {}'.format(ret_str, ret_extra))
        if (self.serial_port.inWaiting()):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('Long response to DC590 read_byte command: {} then {}'.format(ret_str, ret_extra))
        return int(ret_str,16)
    def _write_byte_nopec(self, addr_7bit, command_code, data8):
        '''SMBus Write Byte Protocol without Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns None.'''
        byteList = [self._write_addr(addr_7bit), command_code, data8]
        write_str = 'sS{}S{}S{}p'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(self.serial_port.inWaiting())
        if (len(ret_str) != 0):
            raise Exception('Response: {} from DC590 write_byte command'.format(ret_str))
    def _receive_byte_nopec(self, addr_7bit):
        '''SMBus Receive Byte Protocol without Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns 8-bit data from slave.'''
        write_str = 'sS{}Rp'.format(self._hex_str(self._read_addr(addr_7bit)))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(2)
        if (len(ret_str) != 2):
            raise Exception('Short response to DC590 receive_byte command, EEPROM Present?: %s' % ret_str)
        if (self.serial_port.inWaiting()):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('Long response to DC590 receive_byte command: {} then {}'.format(ret_str, ret_extra))
        return int(ret_str,16)  
    def _send_byte_nopec(self, addr_7bit, data8):
        '''SMBus Send Byte Protocol without Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns None.'''
        byteList = [self._write_addr(addr_7bit), data8]
        write_str = 'sS{}S{}p'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(self.serial_port.inWaiting())
        if (len(ret_str) != 0):
            raise Exception('Response: {} from DC590 send_byte command'.format(ret_str))
    def _alert_response_nopec(self):
        '''SMBus Send Byte Protocol without Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns None if no response to ARA, otherwise the 7-bit device address provided by the slave
        transmit device is placed in the 7 most significant bits of the byte. The eighth bit (R/W̅, LSB) is zero in this implementation.'''
        write_str = 'sS{}Rp'.format(self._hex_str(self._read_addr(0x0C)))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(2)
        if (len(ret_str) != 2):
            raise Exception('Short response to DC590 ARA command')
        if (self.serial_port.inWaiting()):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            if ret_str[0] != 'N':
                raise Exception('Long response to DC590 ARA command: {} then {}'.format(ret_str, ret_extra))
        if ret_str[0] == 'N': #Failed acknowledge == no ARA respondent.
            return None
        return int(ret_str,16) >> 1  
    def _read_word_pec(self, addr_7bit, command_code):
        '''SMBus Read Word Protocol with Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns 16-bit data from slave.'''
        byteList = [self._write_addr(addr_7bit), command_code, self._read_addr(addr_7bit)]
        write_str = 'sS{}S{}sS{}QQRp'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(6)
        if (len(ret_str) != 6):
            raise Exception('Short response to DC590 read_word_pec command, EEPROM Present?: %s' % ret_str)
        byteList.append(int(ret_str[0:2],16))
        byteList.append(int(ret_str[2:4],16))
        if int(ret_str[4:6],16) != self._pec(byteList):
            raise Exception('DC590 read_word_pec command failed PEC')
        if ('N' in ret_str):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('DC590 read_word_pec failed acknowledge: {} then {}'.format(ret_str, ret_extra))
        if ('X' in ret_str):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('DC590 EEPROM detection failed, communications disabled: {} then {}'.format(ret_str, ret_extra))
        if (self.serial_port.inWaiting()):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('Long response to DC590 read_word_pec command: {} then {}'.format(ret_str, ret_extra))
        return self._word(lowByte=int(ret_str[0:2],16), highByte=int(ret_str[2:4],16))
    def _write_word_pec(self, addr_7bit, command_code, data16):
        '''SMBus Write Word Protocol with Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns None.'''
        byteList = [self._write_addr(addr_7bit), command_code, self._lowByte(data16), self._highByte(data16)]
        byteList.append(self._pec(byteList))
        write_str = 'sS{}S{}S{}S{}S{}p'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(self.serial_port.inWaiting())
        if (len(ret_str) != 0):
            raise Exception('Response: {} from DC590 write_word_pec command'.format(ret_str))
    def _read_byte_pec(self, addr_7bit, command_code):
        '''SMBus Read Byte Protocol with Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns 8-bit data from slave.'''
        byteList = [self._write_addr(addr_7bit), command_code, self._read_addr(addr_7bit)]
        write_str = 'sS{}S{}sS{}QRp'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(4)
        if (len(ret_str) != 4):
            raise Exception('Short response to DC590 read_byte_pec command, EEPROM Present?: %s' % ret_str)
        byteList.append(int(ret_str[0:2],16))
        if int(ret_str[2:4],16) != self._pec(byteList):
            raise Exception('DC590 read_byte_pec command failed PEC')
        if ('N' in ret_str):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('DC590 read_byte_pec failed acknowledge: {} then {}'.format(ret_str, ret_extra))
        if ('X' in ret_str):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('DC590 EEPROM detection failed, communications disabled: {} then {}'.format(ret_str, ret_extra))
        if (self.serial_port.inWaiting()):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('Long response to DC590 read_byte_pec command: {} then {}'.format(ret_str, ret_extra))
        return int(ret_str[0:2],16)
    def _write_byte_pec(self, addr_7bit, command_code, data8):
        '''SMBus Write Byte Protocol with Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns None.'''
        byteList = [self._write_addr(addr_7bit), command_code, data8]
        byteList.append(self._pec(byteList))
        write_str = 'sS{}S{}S{}S{}p'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(self.serial_port.inWaiting())
        if (len(ret_str) != 0):
            raise Exception('Response: {} from DC590 write_byte_pec command'.format(ret_str))
    def _receive_byte_pec(self, addr_7bit):
        '''SMBus Receive Byte Protocol with Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns 8-bit data from slave.'''
        byteList = [self._read_addr(addr_7bit)]
        write_str = 'sS{}QRp'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(4)
        if (len(ret_str) != 4):
            raise Exception('Short response to DC590 receive_byte_pec command, EEPROM Present?: %s' % ret_str)
        byteList.append(int(ret_str[0:2],16))
        if int(ret_str[2:4],16) != self._pec(byteList):
            raise Exception('DC590 receive_byte_pec command failed PEC')
        if (self.serial_port.inWaiting()):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            raise Exception('Long response to DC590 receive_byte_pec command: {} then {}'.format(ret_str, ret_extra))
        return int(ret_str[0:2],16)  
    def _send_byte_pec(self, addr_7bit, data8):
        '''SMBus Send Byte Protocol with Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns None.'''
        byteList = [self._write_addr(addr_7bit), data8]
        byteList.append(self._pec(byteList))
        write_str = 'sS{}S{}S{}p'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(self.serial_port.inWaiting())
        if (len(ret_str) != 0):
            raise Exception('Response: {} from DC590 send_byte_pec command'.format(ret_str))
    def _alert_response_pec(self):
        '''SMBus Send Byte Protocol with Packet Error Checking.
        Slave device address specified in 7-bit format.
        Returns None if no response to ARA, otherwise the 7-bit device address provided by the slave
        transmit device is placed in the 7 most significant bits of the byte. The eighth bit (R/W̅, LSB) is zero in this implementation.'''
        byteList = [self._read_addr(0x0C)]
        write_str = 'sS{}QRp'.format(*map(self._hex_str, byteList))
        self.serial_port.write(write_str)
        ret_str = self.serial_port.read(4)
        if (len(ret_str) != 4):
            raise Exception('Short response to DC590 ARA_pec command')
        if (self.serial_port.inWaiting()):
            time.sleep(self._serial_wait)
            ret_extra = self.serial_port.read(self.serial_port.inWaiting())
            if ret_str[0] != 'N':
                raise Exception('Long response to DC590 ARA_PEC command: {} then {}'.format(ret_str, ret_extra))
        if ret_str[0] == 'N': #Failed acknowledge == no ARA respondent.
            return None
        byteList.append(int(ret_str[0:2],16))
        if int(ret_str[2:4],16) != self._pec(byteList):
            raise Exception('DC590 ARA_pec command failed PEC')
        return int(ret_str[0:2],16) >> 1  

import sqlite3, datetime, time
class logger(object):
    def __init__(self, chip, db_filename="data_log.sqlite"):
        '''Data logger stores memory dump(s) of the LTC2937 to a SQLite database file.
        chip is an instance of the LTC2937 class.
        A new database table will be automatically created with the current timestamp.
        '''
        self.chip = chip
        self.conn = sqlite3.connect(db_filename)
        self.cursor = self.conn.cursor()
        column_names = "rowid INTEGER PRIMARY KEY, datetime DATETIME, "
        for bit_field in sorted(self.chip.keys()):
            column_names += "{},".format(bit_field)
        column_names = column_names[:-1]
        self.q = ("?," * (len(self.chip.keys())+2))[:-1]
        self.table_name = "log_{}".format(datetime.datetime.now().strftime("%Y_%m_%d_%H_%M_%S"))
        self.cursor.execute("CREATE TABLE {} ({});".format(self.table_name, column_names))
    def log(self):
        '''Store single dump of the LTC2937 memory to the database.'''
        mem = [('rowid', None), ('datetime', datetime.datetime.utcnow().strftime('%Y-%m-%dT%H:%M:%S.%fZ'))] + sorted(self.chip.items(),key=lambda (k,v): k)
        keys, values = zip(*mem)
        sql = "INSERT INTO {} {} VALUES ({});".format(self.table_name, str(tuple(keys)), self.q)
        self.cursor.execute(sql,values)
        self.conn.commit()
        return dict(mem)
    def interval_log(self, interval_seconds, max_rows=None):
        '''Repeatedly store dump of the LTC2937 memory to the database.
        Memory will be recorded once per interval_seconds.
        Data will be recorded max_rows times.  If set to None, recording will continue indefinitely.
        '''
        rowcount = 0
        while True:
            begin_time = time.time()
            rowcount += 1
            if max_rows is not None and rowcount > max_rows:
                return
            self.log()
            print("Memory stored to row {} at {}".format(rowcount-1,datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")))
            time.sleep(max(interval_seconds - (time.time()-begin_time),0))

if __name__ == '__main__':
    print('\n\nPlease see the file "example.py"\n\n')

