# LTC2937 Voltage Readback Example

####
# Python code example for running DC590b emulation on Linduino, or straight into DC590b or DC1613a, talking to LTC2937 demoboard.
# The code accesses the LTC2937 register definition contained in the XML file. The SAR_demo example uses the OV comparators
# on each voltage monitor channel to perform a SAR conversion on the input voltages. The break_point_stepper_demo demonstrates
# the use of break-point features to single step the on board power supplies. Choose the demo at the bottom.
# The code uses the I2C mode for sequencing to enable/disable the six on board regulators. To exit, press ctrl-c
# Global address is used for communication. The COM port number (below) may have to be changed to match hardware.

#### A Note about command names:
###     To read or write the whole word of a LTC2937 Command that is bit-packed (multiple bit fields defined),
###     add '_CMD' to the command name (example: V_THRESHOLD_1_CMD). For the few commands that do not have bit-packed
###     words, do not use the '_CMD' extension. Those commands are:
###     SPECIAL_LOT, CLEAR_ALERTB, CLEAR, STORE, RESTORE, DEVICE_ID

#### To execute the Alert Response Protocol, use ARA

import LTC2937

dc590b_serial_port = 'COM4' #Modify this line as appropriate (for Linux, '/dev/ttyXX', etc)
dc590b = LTC2937.ltc_dc590b_interface(serial_port = dc590b_serial_port, PEC = False)

LTC2937.LTC2937.STORE = lambda self: dc590b.send_byte(self.addr_7bit, 0x2C)
LTC2937.LTC2937.RESTORE = lambda self: dc590b.send_byte(self.addr_7bit, 0x2D)
LTC2937.LTC2937.CLEAR = lambda self: dc590b.send_byte(self.addr_7bit, 0x2E)

LTC2937.LTC2937.ARA = lambda self: dc590b.alert_response()

chip = LTC2937.LTC2937(read_function = dc590b.read_word, write_function = dc590b.write_word, verbose = False)
chip.set_device_address(0x36)

channel_number = 1
        
import sys
#sys.exit()

chip_id = chip.DEVICE_ID
print 'Device ID = ',hex(chip_id),('\n')

#check for LTC2937 hardware lock, notify user if change required

if chip['hw_lock_bit'] == 'UNLOCKED':
    print 'Hardware is unlocked'
    
else:
    print 'Hardware is locked, move WP jumper to DIS'
    print ('\n')
    sys.exit()
    
#check for LTC2937 software lock, if locked, then unlock

if chip['sw_lock_bit'] == 'UNLOCKED':
    print 'Software bit is unlocked'
    print ('\n')
else:
    print 'Software bit is locked'
    chip.sw_lock_bit = 'UNLOCKED'
    print 'Software bit is now unlocked'
    print ('\n')
    
# if demo-board is sequenced up, bring it down before configuring
# this demo, then clear faults and status bits. Some LED's may change state.

if chip['local_seq_status'] <> 'SEQUENCE_DOWN_DONE':
    chip['bp_enable'] = 'BREAK_POINT_NOT_ENABLED'   #if break point on, turn it off
    chip['i2c_on_off_mask'] = 'I2C_LISTEN'
    chip['i2c_on_off'] = 'SEQUENCE_DOWN'
    chip['on_input_mask'] = 'ON_PIN_IGNORE'
else:
    chip['on_input_mask'] = 'ON_PIN_IGNORE'

# if supplies were on, wait until they decay to discharge threshold
if chip['discharge'] == 'ONE_OR_MORE_SEQUENCED_CHANNELS_NOT_DISCHARGED':
    print 'One or more sequenced channels are not discharged',
while chip['discharge'] == 'ONE_OR_MORE_SEQUENCED_CHANNELS_NOT_DISCHARGED':
    print '.',

print ('\n')

# clear any former alerts and then clear device status and reset state machines
#chip.ARA()

chip.CLEAR_ALERTB
chip.CLEAR()


#### set up device with demo-board defaults in case they have been changed

# SEQ_UP_POSITION_n
chip['SEQ_UP_POSITION_1_CMD'] = 1
chip['SEQ_UP_POSITION_2_CMD'] = 2
chip['SEQ_UP_POSITION_3_CMD'] = 3
chip['SEQ_UP_POSITION_4_CMD'] = 4
chip['SEQ_UP_POSITION_5_CMD'] = 5
chip['SEQ_UP_POSITION_6_CMD'] = 6

# TON_TIMERS_n
for channel_number in xrange(1,7):
    chip['ton_max_{}'.format(channel_number)] = '655ms' #set max rise time
    chip['ton_delay_{}'.format(channel_number)] = 0.328 #set delay time (328ms)

# V_RANGE
chip.v1_range = 'HIGH_RANGE'  #high range for channel 1
chip.set_active_format('ov_threshold_1','Volts_to_TE_HR')   #using pre-defined formatting
chip.set_active_format('uv_threshold_1','Volts_to_TE_HR')   #using pre-defined formatting

for channel_number in xrange(2,7):
    chip['v{}_range'.format(channel_number)] = 'LOW_RANGE' #low range for channels 2-6
    chip.set_active_format('ov_threshold_{}'.format(channel_number),'Volts_to_TE_LR')   #using pre-defined formatting
    chip.set_active_format('uv_threshold_{}'.format(channel_number),'Volts_to_TE_LR')   #using pre-defined formatting
    
# V_THRESHOLD_n         #set OV and UV thresholds for the demo-board power supplies
chip['ov_threshold_1'] = 3.3*1.1    #int(50*(3.3*1.1 - 0.9))    # +10% OV
chip['ov_threshold_2'] = 2.5*1.1    #int(100*(2.5*1.1 - 0.45))  # +10% OV   
chip['ov_threshold_3'] = 2.0*1.1    #int(100*(2.0*1.1 - 0.45))  # +10% OV    
chip['ov_threshold_4'] = 1.8*1.1    #int(100*(1.8*1.1 - 0.45))  # +10% OV    
chip['ov_threshold_5'] = 1.5*1.1    #int(100*(1.5*1.1 - 0.45))  # +10% OV    
chip['ov_threshold_6'] = 1.2*1.1    #int(100*(1.2*1.1 - 0.45))  # +10% OV     

chip['uv_threshold_1'] = 3.3*0.95   #int(50*(3.3*0.95 - 0.9))    # -5% UV   
chip['uv_threshold_2'] = 2.5*0.95   #int(100*(2.5*0.95 - 0.45))  # -5% UV     
chip['uv_threshold_3'] = 2.0*0.95   #int(100*(2.0*0.95 - 0.45))  # -5% UV      
chip['uv_threshold_4'] = 1.8*0.95   #int(100*(1.8*0.95 - 0.45))  # -5% UV      
chip['uv_threshold_5'] = 1.5*0.95   #int(100*(1.5*0.95 - 0.45))  # -5% UV       
chip['uv_threshold_6'] = 1.2*0.95   #int(100*(1.2*0.95 - 0.45))  # -5% UV       

# RSTB_CONFIG
chip['rstb_delay'] = '200ms'
chip['sv_fault_enable'] = 'SUPERVISOR_FAULT_PULLS_FAULTB'

for channel_number in xrange(1,7):
    chip['v{}_ov_enable'.format(channel_number)] = 'NOT_ENABLED'  #remove OV comparators from RSTB participation
    chip['v{}_uv_enable'.format(channel_number)] = 'ENABLED'  #include UV comparators in RSTB participation

# SEQ_DOWN_POSITION_n
chip['SEQ_DOWN_POSITION_1_CMD'] = 6
chip['SEQ_DOWN_POSITION_2_CMD'] = 5
chip['SEQ_DOWN_POSITION_3_CMD'] = 4
chip['SEQ_DOWN_POSITION_4_CMD'] = 3
chip['SEQ_DOWN_POSITION_5_CMD'] = 2
chip['SEQ_DOWN_POSITION_6_CMD'] = 1

# TOFF_TIMERS_n
for channel_number in xrange(1,7):
    chip['toff_max_{}'.format(channel_number)] = '164ms'  #set max fall time
    chip['toff_delay_{}'.format(channel_number)] = 0  #set delay time (0ms)

# FAULT_RESPONSE
chip['faultb_mask'] = 'IGNORE_EXTERNAL_FAULTB_PULLDOWN'
chip['freeze'] = 'DO_NOT_FREEZE_ON_FAULT'
chip['retry_delay'] = '1_64s'
chip['fault_response_action'] = 'DELAYED_RETRY'
chip['retry_number'] = 'UNLIMITED_RETRIES'

# BREAK_POINT
chip['bp_enable'] = 'BREAK_POINT_NOT_ENABLED'
chip['bp_value'] = 0
    
# ON_OFF_CONTROL
chip['i2c_margin'] = 'RSTB_OPERATES_NORMALLY'
chip['discharge_start'] = 'DISCHARGE_NOT_REQUIRED'
#chip['i2c_on_off'] = 'SEQUENCE_UP'  #start sequencing in the demo routine, once everything is configured.
chip['i2c_on_off_mask'] = 'I2C_LISTEN'
#chip['on_input_mask'] = 'ON_PIN_LISTEN' #don't use the demo-board push-button input, masked off earlier
chip['on_polarity'] = 'ACTIVE_HIGH'
chip['seq_down_qual'] = 'TIME_BASED'    

chip['faultb_mask'] = 'RESPOND_TO_EXTERNAL_FAULTB_PULLDOWN'

###################################################################       

def SAR_demo():

    while True:
    
# make some changes to demo-board setup to create 6-channel SAR conversions using the OV comparators
# print out the six converted voltages to the screen

        chip['i2c_on_off'] = 'SEQUENCE_UP' #start sequencing up

        for channel_number in xrange(1,7):
            
            channel_name = 'ov_threshold_{}'.format(channel_number)
            channel_format = chip.get_active_format(channel_name)
            chip.set_active_format(channel_name,None) #use threshold encoding format during SAR conversion
            chip[channel_name] = 128  #initialize threshold encoding to center of range
            
            sar_guess = 64
            
            while sar_guess >= 1:
                
                if chip['v{}_ov_status'.format(channel_number)] == 'HIGH_LIMIT_VIOLATION':
                    chip[channel_name] += sar_guess
                else:
                    chip[channel_name] -= sar_guess
                    
                sar_guess = sar_guess/2 #divide sar_guess by 2
                    
            if chip['v{}_ov_status'.format(channel_number)] != 'HIGH_LIMIT_VIOLATION':
                chip[channel_name] -= 1
            
            chip.set_active_format(channel_name,channel_format) #switch register format back to Volts after SAR conversion
        
        print 'V1 = {}V '.format((chip.ov_threshold_1)),    #/50.00) + 0.9),    #format switch takes care of encoding to volts
        print 'V2 = {}V '.format((chip.ov_threshold_2)),    #/100.00) + 0.45),  #format switch takes care of encoding to volts
        print 'V3 = {}V '.format((chip.ov_threshold_3)),    #/100.00) + 0.45),  #format switch takes care of encoding to volts
        print 'V4 = {}V '.format((chip.ov_threshold_4)),    #/100.00) + 0.45),  #format switch takes care of encoding to volts
        print 'V5 = {}V '.format((chip.ov_threshold_5)),    #/100.00) + 0.45),  #format switch takes care of encoding to volts
        print 'V6 = {}V '.format((chip.ov_threshold_6)),    #/100.00) + 0.45),  #format switch takes care of encoding to volts
        print ('\n')  

###################################################################

###################################################################

def break_point_stepper_demo():

    chip['bp_enable'] = 'BREAK_POINT_ENABLED'
    chip['i2c_on_off'] = 'SEQUENCE_UP' #start sequencing up

    while True:

        while chip.sp_count < 7:
        
            print 'sequence position =',[chip.sp_count],('\n') #read sequence position from internal counter

            raw_input('Press Enter to advance')
            
            chip['bp_value'] += 1   #advance break point
        
        chip['bp_value'] = 0
        chip['i2c_on_off'] = 'SEQUENCE_DOWN' #start sequencing down
        
        while chip.sp_count < 7:
        
            print 'sequence position =',[chip.sp_count],('\n') #read sequence position from internal counter

            raw_input('Press Enter to advance')
                
            chip['bp_value'] += 1   #advance break point
    
        chip['bp_value'] = 0
        chip['i2c_on_off'] = 'SEQUENCE_UP' #start sequencing up

###################################################################

if __name__ == "__main__":
    
    SAR_demo()
    #break_point_stepper_demo()  #to change demos, un-comment this line and comment out the SAR_demo