/*!
LTC2937: Programmable Six Channel Sequencer and Voltage Supervisor with EEPROM

@verbatim
The LTC®2937 is a six-channel power supply sequencer and voltage supervisor.
Supplies are ENABLED or disabled with precise user controlled order and time
spacing. To detect power supply output faults during sequencing and monitoring,
the LTC2937 accurately monitors supply turn-ON/-OFF delays and output voltage
levels. In the event of a fault, response actions include complete power supply
shutdown and optional restarts. Root cause of power faults are logged to EEPROM.
For systems with high supply count, a simple single wire connection between
multiple LTC2937 devices allows sequencing expansion to 300 supplies. After
successful sequencing and supply voltage stabilization, the reset output pulls
high to initiate microprocessor or other system activity. To accommodate supply
margin testing, the reset output can be disabled. Upon supply turn-OFF,
integrated current sources are available as needed to discharge slowly decaying
supplies. Configuration EEPROM supports autonomous operation without software.
@endverbatim

http://www.linear.com/product/LTC2937

http://www.linear.com/product/LTC2937#demoboards

REVISION HISTORY
$Revision$
$Date$

Copyright (c) 2016, Linear Technology Corp.(LTC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1.  Redistributions  of source code must retain the above copyright notice, this
    list  of conditions and the following disclaimer.

2.  Redistributions  in  binary  form must reproduce the above copyright notice,
    this  list of conditions and  the following disclaimer in the  documentation
    and/or other materials provided with the distribution.

THIS  SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY  EXPRESS  OR  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES   OF  MERCHANTABILITY  AND  FITNESS  FOR  A  PARTICULAR  PURPOSE  ARE
DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY  DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING,  BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS  OF  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT  LIABILITY,  OR TORT
(INCLUDING  NEGLIGENCE  OR  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The  views and conclusions contained in the software and documentation are those
of  the authors and should not be interpreted as representing official policies,
either expressed or implied, of Linear Technology Corp.

The Linear Technology Linduino is not affiliated with the official Arduino team.
However,  the Linduino is only possible because of the Arduino team's commitment
to   the   open-source   community.   Please,  visit  http://www.arduino.cc  and
http://store.arduino.cc,  and consider  a purchase  that  will  help fund  their
ongoing work.

Generated on: 2016-07-27
*/


/*! @file
 *  @ingroup LTC2937
 *  @brief LTC2937 communication library core header file defining
 *  prototypes, data structures and constants used by LTC2937.c
 *
 *  Functions  matching  the  prototypes  of  @ref  smbus_write_register and @ref
 *  smbus_read_register  must  be  provided  to this API. They will implement the
 *  SMBus  read  and write transactions on your hardware. If the register size of
 *  the  LTC2937  is 8 bits, functions should be provided that implement SMBus
 *  read  byte and write byte. If the register size of the LTC2937 is 16 bits,
 *  functions  should  be provided that implement SMBus read word and write word.
 *  smbus_read_register  needs  to  store the value read from the LTC2937 into
 *  the  variable  data.  Both  functions  should return 0 on success and a non-0
 *  error  code  on  failure.  The  API functions will return your error codes on
 *  failure and a 0 on success.
 */

#ifndef LTC2937_H_
#define LTC2937_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "LTC2937_reg_defs.h"
#include "LTC2937_formats.h"
#include <stdint.h>
#include <stddef.h>

  // Type declarations
  /*! Hardware port information. Modify as needed for local hardware
  requirements. Available to user supplied read and write functions. */
  typedef struct
  {
    int file_descriptor; //!< Linux SMBus file handle
  } port_configuration_t;
  /*! Prototype of user supplied SMBus write_byte or write_word function. Should return 0 on success and a non-0 error code on failure. */
  typedef int (*smbus_write_register)(uint8_t addr, //!< Target IC's SMBus address
                                      uint8_t command_code, //!< Command code to be written to
                                      uint16_t data, //!< Data to be written
                                      port_configuration_t *port_configuration //!< Pointer to a user supplied port_configuration struct
                                     );
  /*! Prototype of user supplied SMBus read_byte or read_word function. Should return 0 on success and a non-0 error code on failure. */
  typedef int (*smbus_read_register)(uint8_t addr, //!< Target IC's SMBus address
                                     uint8_t command_code, //!< command code to be read from
                                     uint16_t *data, //!< Pointer to data destination
                                     port_configuration_t *port_configuration //!< Pointer to a user supplied port_configuration struct
                                    );
  typedef void *LTC2937;
  /*! Information required to access hardware SMBus port */
  typedef struct
  {
    uint8_t addr; //!< Target IC's SMBus address
    smbus_read_register read_register; //!< Pointer to a user supplied smbus_read_register function
    smbus_write_register write_register; //!< Pointer to a user supplied smbus_write_register function
    port_configuration_t *port_configuration; //!< Pointer to a user supplied port_configuration struct
  } LTC2937_chip_cfg_t;

  // function declarations
  /*! Returns a pointer to a LTC2937 structure used by LTC2937_write_register and LTC2937_read_register */
  LTC2937 LTC2937_init(LTC2937_chip_cfg_t *cfg //!< Information required to access hardware SMBus port
                      );
  /*! Function to modify a bit field within a register while preserving the unaddressed bit fields */
  int LTC2937_write_register(LTC2937 chip_handle, //!< Struct returned by LTC2937_init
                             uint16_t registerinfo, //!< Bit field name from LTC2937_regdefs.h
                             uint16_t data //!< Data to be written
                            );
  /*! Retrieves a bit field data into *data. Right shifts the addressed portion down to the 0 position */
  int LTC2937_read_register(LTC2937 chip_handle, //!< Struct returned by LTC2937_init
                            uint16_t registerinfo, //!< Register name from LTC2937_regdefs.h
                            uint16_t *data //!< Pointer to the data destination
                           );
  /*! Multiple LTC2937 use.
    Multiple LTC2937s can be used with this API. Each one must be initialized.
    The LTC2937_init requires some memory for each LTC2937. This memory can
    be  statically  allocated  by  defining  MAX_NUM_LTC2937_INSTANCES  to the
    number of LTC2937s required. Alternatively it can be dynamically allocated
    by  defining  LTC2937_USE_MALLOC.  The  default  is  to not use malloc and
    statically allocate one LTC2937.
  */
#ifndef MAX_NUM_LTC2937_INSTANCES
#define MAX_NUM_LTC2937_INSTANCES 1
#endif
#ifdef __cplusplus
}
#endif
#endif /* LTC2937_H_ */
