/*
LTC2937: Programmable Six Channel Sequencer and Voltage Supervisor with EEPROM

@verbatim
The LTC®2937 is a six-channel power supply sequencer and voltage supervisor.
Supplies are ENABLED or disabled with precise user controlled order and time
spacing. To detect power supply output faults during sequencing and monitoring,
the LTC2937 accurately monitors supply turn-ON/-OFF delays and output voltage
levels. In the event of a fault, response actions include complete power supply
shutdown and optional restarts. Root cause of power faults are logged to EEPROM.
For systems with high supply count, a simple single wire connection between
multiple LTC2937 devices allows sequencing expansion to 300 supplies. After
successful sequencing and supply voltage stabilization, the reset output pulls
high to initiate microprocessor or other system activity. To accommodate supply
margin testing, the reset output can be disabled. Upon supply turn-OFF,
integrated current sources are available as needed to discharge slowly decaying
supplies. Configuration EEPROM supports autonomous operation without software.
@endverbatim

http://www.linear.com/product/LTC2937

http://www.linear.com/product/LTC2937#demoboards

REVISION HISTORY
$Revision$
$Date$

Copyright (c) 2016, Linear Technology Corp.(LTC)
All rights reserved.

Linear Technology Confidential - For Customer Use Only

THIS  SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY  EXPRESS  OR  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES   OF  MERCHANTABILITY  AND  FITNESS  FOR  A  PARTICULAR  PURPOSE  ARE
DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY  DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING,  BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS  OF  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT  LIABILITY,  OR TORT
(INCLUDING  NEGLIGENCE  OR  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


/*! @file
 *  @ingroup LTC2937
 *  @brief LTC2937 communication library example
 *
 *  This is an example program to demonstrate use of the API. The
 *  my_write_register and my_read_register functions simply display data and
 *  return constant data to allow compiling and demonstration without hardware.
 */

#include "LTC2937_SDK.h"
#include <stdio.h>
#include <stdlib.h>

/*!
This  is  an  example  of an smbus_read_register function. In a real application
this  function  would  do  an SMBus read_word transaction of slave address
addr,  and  command  code command_code, and place the returned data in *data. It
should  return  a  0  for success and a non-zero error code for failure. The API
functions  will  return  this error code in the event of an error. If you do not
need error handling, it should simply always return 0.
*/
int my_read_register(uint8_t addr, //!< Target IC's SMBus address
                     uint8_t command_code, //!< Command Code to be read from
                     uint16_t *data, //!< Pointer to data destination
                     port_configuration_t *port_configuration //!< Pointer to port configuration struct
                    )
{
  printf("Read word Transaction:\n");
  printf("\tAddr: %d\n",addr);
  printf("\tCommand_code: %d\n",command_code);
  *data = rand() % (1<<16);
  printf("\tRaw read fake data: %d\n",*data);
  return 0;
}

/*!
This  is  an  example of an smbus_write_register function. In a real application
this  function  would  do an SMBus write_word transaction of slave address
addr,  and  command  code  command_code,  writing data. It should return a 0 for
success  and  a  non-zero  error code for failure. The API functions will return
this  error code in the event of an error. If you do not need error handling, it
should simply always return 0.
*/
int my_write_register(uint8_t addr, //!< Target IC's SMBus address
                      uint8_t command_code, //!< Command Code to be written to
                      uint16_t data, //!< Data to be written
                      port_configuration_t *port_configuration //!< Pointer to port configuration struct
                     )
{
  printf("Write word Transaction:\n");
  printf("\tAddr: %d\n",addr);
  printf("\tCommandCode: %d\n",command_code);
  printf("\tRaw Data: %d\n",data);
  return 0;
}

int main(void)
{
  uint16_t data;
  LTC2937_chip_cfg_t cfg =
  {
    .addr = LTC2937_ADDR_36,
    .read_register = my_read_register,
    .write_register = my_write_register,
    //!< Pointer to port configuration struct not used without physical port
    //.port_configuration = &pc
  };

  printf("Initializing LTC2937\n");
  LTC2937 chip = LTC2937_init(&cfg);

  /*! the API functions can be used to read and write individual bit fields
  within a command code */
  printf("Using Read/Modify/Write Register for the LTC2937_V1_OV_SV_FAULT bit field\n");
  LTC2937_write_register(chip, LTC2937_V1_OV_SV_FAULT, 0x1);
  printf("Using Read Register for the LTC2937_V1_OV_SV_FAULT bit field\n");
  LTC2937_read_register(chip, LTC2937_V1_OV_SV_FAULT, &data);
  printf("\tRead: %d\n",data);

  /*! the API functions can also be used to read and write whole command codes */
  printf("Using Read/Modify/Write Register for the LTC2937_SEQ_UP_POSITION_5_CMD command code\n");
  LTC2937_write_register(chip, LTC2937_SEQ_UP_POSITION_5_CMD, 0x1);
  printf("Using Read Register for the LTC2937_SEQ_UP_POSITION_5_CMD command code\n");
  LTC2937_read_register(chip, LTC2937_SEQ_UP_POSITION_5_CMD, &data);
  printf("\tRead: %d\n",data);


  return 0;
}
