/*!
LTC2937: Programmable Six Channel Sequencer and Voltage Supervisor with EEPROM

@verbatim
The LTC®2937 is a six-channel power supply sequencer and voltage supervisor.
Supplies are ENABLED or disabled with precise user controlled order and time
spacing. To detect power supply output faults during sequencing and monitoring,
the LTC2937 accurately monitors supply turn-ON/-OFF delays and output voltage
levels. In the event of a fault, response actions include complete power supply
shutdown and optional restarts. Root cause of power faults are logged to EEPROM.
For systems with high supply count, a simple single wire connection between
multiple LTC2937 devices allows sequencing expansion to 300 supplies. After
successful sequencing and supply voltage stabilization, the reset output pulls
high to initiate microprocessor or other system activity. To accommodate supply
margin testing, the reset output can be disabled. Upon supply turn-OFF,
integrated current sources are available as needed to discharge slowly decaying
supplies. Configuration EEPROM supports autonomous operation without software.
@endverbatim

http://www.linear.com/product/LTC2937

http://www.linear.com/product/LTC2937#demoboards

REVISION HISTORY
$Revision$
$Date$

Copyright (c) 2016, Linear Technology Corp.(LTC)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

1.  Redistributions  of source code must retain the above copyright notice, this
    list  of conditions and the following disclaimer.

2.  Redistributions  in  binary  form must reproduce the above copyright notice,
    this  list of conditions and  the following disclaimer in the  documentation
    and/or other materials provided with the distribution.

THIS  SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY  EXPRESS  OR  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES   OF  MERCHANTABILITY  AND  FITNESS  FOR  A  PARTICULAR  PURPOSE  ARE
DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY  DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING,  BUT  NOT  LIMITED  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS  OF  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY  THEORY  OF  LIABILITY,  WHETHER  IN  CONTRACT,  STRICT  LIABILITY,  OR TORT
(INCLUDING  NEGLIGENCE  OR  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The  views and conclusions contained in the software and documentation are those
of  the authors and should not be interpreted as representing official policies,
either expressed or implied, of Linear Technology Corp.

The Linear Technology Linduino is not affiliated with the official Arduino team.
However,  the Linduino is only possible because of the Arduino team's commitment
to   the   open-source   community.   Please,  visit  http://www.arduino.cc  and
http://store.arduino.cc,  and consider  a purchase  that  will  help fund  their
ongoing work.

Generated on: 2016-07-27
*/


/*! @file
 *  @ingroup LTC2937
 *  @brief LTC2937 library file defining data conversion macros and constants used
 *  by LTC2937.c
 *
 *
 *  This file contains macros and constants which can be used to represent real-world
 *  values in the source code, with automatic compile-time conversion to LTC2937
 *  internal register scaling.
 *
 *  Passing runtime variables to these macros should be avoided as it will likely result
 *  in runtime calculations accompanied by associated processor loading and memory usage.
 */

#ifndef LTC2937_FORMATS_H_
#define LTC2937_FORMATS_H_

/*! @name Use the macros below to convert from real world to LTC2937 referenced numbers. */
/*! convert an Enable Delay to 13-bit integer
 *   - Used with Bit Fields: ton_delay_1, ton_delay_2, ton_delay_3, ton_delay_4, ton_delay_5, ton_delay_6, toff_delay_1, toff_delay_2, toff_delay_3, toff_delay_4, toff_delay_5, toff_delay_6.
 */
#define LTC2937_DELAY_TIME_TO_INTEGER(x) __LTC2937_LINE__((0), (80E-6), (0), (1), x)

/*! convert a voltage value to Threshold Encoding for High Range
 *   - Used with Bit Fields: ov_threshold_1, uv_threshold_1, ov_threshold_2, uv_threshold_2, ov_threshold_3, uv_threshold_3, ov_threshold_4, uv_threshold_4, ov_threshold_5, uv_threshold_5, ov_threshold_6, uv_threshold_6.
 */
#define LTC2937_VOLTS_TO_TE_HR(x) __LTC2937_LINE__((0.9), (0.9 + 0.02), (0), (1), x)

/*! convert a voltage value to Threshold Encoding for Low Range
 *   - Used with Bit Fields: ov_threshold_1, uv_threshold_1, ov_threshold_2, uv_threshold_2, ov_threshold_3, uv_threshold_3, ov_threshold_4, uv_threshold_4, ov_threshold_5, uv_threshold_5, ov_threshold_6, uv_threshold_6.
 */
#define LTC2937_VOLTS_TO_TE_LR(x) __LTC2937_LINE__((0.45), (0.45 + 0.01), (0), (1), x)

/*! convert a voltage value to Threshold Encoding for Adjustable Range
 *   - Used with Bit Fields: ov_threshold_1, uv_threshold_1, ov_threshold_2, uv_threshold_2, ov_threshold_3, uv_threshold_3, ov_threshold_4, uv_threshold_4, ov_threshold_5, uv_threshold_5, ov_threshold_6, uv_threshold_6.
 */
#define LTC2937_VOLTS_TO_TE_AR(x) __LTC2937_LINE__((0.18), (0.18 + 0.004), (0), (1), x)

/*! @name Private macros for use by formats above.
 */
#define __LTC2937_LINE__(x0,x1,y0,y1,x) (uint16_t)((y0) + ((y1) - (y0))/((x1) - (x0)) * ((x) - (x0)) + 0.5)
#define __LTC2937_BETWEEN_INCLUSIVE__(x0,x1,x) (((x) > (x0) && (x) < (x1)) || (x) == (x0) || (x) == (x1) ? 1 : 0)


#endif /* LTC2937_FORMATS_H_ */
